package controller;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

/**
 * Class for scaling images.
 * Class can be found here: http://frickelblog.wordpress.com/2009/06/08/fast-image-scaling-in-java/
 * @author Johann Houszka 0625523
 */
/**
 * @author Johann Houszka 0625523
 *
 */
public class FastImageScaler {

    /**
     * Scales Image.
     * @param img
     * @param d
     * @return
     */
    public BufferedImage scaleImage(BufferedImage img, Dimension d) {
        img = scaleByHalf(img, d);
        img = scaleExact(img, d);
        return img;
    }

    /**
     * Scales image to half of the size.
     * @param img
     * @param d
     * @return
     */
    private BufferedImage scaleByHalf(BufferedImage img, Dimension d) {
        int w = img.getWidth();
        int h = img.getHeight();
        float factor = getBinFactor(w, h, d);

        // make new size
        w *= factor;
        h *= factor;
        if(w<=0) w = 1;
        if(h<=0) h = 1;
        BufferedImage scaled = new BufferedImage(w, h,
                BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = scaled.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        g.drawImage(img, 0, 0, w, h, null);
        g.dispose();
        return scaled;
    }

    /**
     * Scales images to an excact size.
     * @param img
     * @param d
     * @return
     */
    private BufferedImage scaleExact(BufferedImage img, Dimension d) {
        float factor = getFactor(img.getWidth(), img.getHeight(), d);

        // create the image
        int w = (int) (img.getWidth() * factor);
        int h = (int) (img.getHeight() * factor);
        if(w<=0) w = 1;
        if(h<=0) h = 1;
        BufferedImage scaled = new BufferedImage(w, h,
                BufferedImage.TYPE_INT_ARGB);

        Graphics2D g = scaled.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.drawImage(img, 0, 0, w, h, null);
        g.dispose();
        return scaled;
    }

    /**
     * Returns the scale-factor.
     * @param width
     * @param height
     * @param dim
     * @return
     */
    float getBinFactor(int width, int height, Dimension dim) {
        float factor = 1;
        float target = getFactor(width, height, dim);
        if (target <= 1) { while (factor / 2 > target) { factor /= 2; }
        } else { while (factor * 2 < target) { factor *= 2; }         }
        return factor;
    }

    
    /**
     * Returns the sale factor.
     * @param width
     * @param height
     * @param dim
     * @return
     */
    float getFactor(int width, int height, Dimension dim) {
        float sx = dim.width / (float) width;
        float sy = dim.height / (float) height;
        return Math.min(sx, sy);
    }
}