package controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.PlayerImage;
import model.PlayerImages;
import model.Subtitle;
import model.Subtitles;
import model.Titles;
import player.Player;
import save.SaveVideo;
import uk.co.caprica.vlcj.filter.swing.SwingFileFilterFactory;
import view.PlayerAboutDialog;
import view.PlayerControlsPanel;
import view.PlayerEffectsPanel;
import view.PlayerMainPanel;
import view.PlayerSaveDialog;
import view.PlayerSettingsPanel;
import view.StatusDialog;
import effects.Effect;

/**
 * Listener class for all UI-elements.
 * @author Johann Houszka, Matthias Eigner
 *
 */
public class PlayerUIListener implements ActionListener {
	private Player player;
	private PlayerMainPanel mainPanel;
	private PlayerControlsPanel controlsPanelMain;
	private PlayerEffectsPanel effectsPanelMain;
	private PlayerSettingsPanel settingsPanelMain;
	private String filepath, filepath_img, videoPath;
	private Subtitles sub;
	public boolean subtitlesLoaded;
	private String logopath;
	public boolean isVideoLoaded;
	private boolean saving;
	private PlayerImages images;
	private PlayerSaveDialog saveDialog;
	private PlayerAboutDialog aboutDialog;
	private StatusDialog statusDialog;
	
	private Titles titles;

	/**
	 * Constructor for this class.
	 * @param Player player
	 * @param PlayerMainPanel mainPanel
	 */
	public PlayerUIListener(Player player, PlayerMainPanel mainPanel) {
		this.player = player;
		this.mainPanel = mainPanel;
		this.controlsPanelMain = null;
		this.effectsPanelMain = null;
		this.settingsPanelMain = null;
		this.subtitlesLoaded = false;
		this.isVideoLoaded = false;
		this.saveDialog = null;
		this.aboutDialog = null;
				
		this.images = new PlayerImages();
		this.titles = new Titles(this.player);
		
	}
	
	/**
	 * Returns the titles of type Titles.
	 * @return Titles title
	 */
	public Titles getTitles() {
		return this.titles;
	}
	
	/**
	 * Returns subtitles of type Subtitles.
	 * @return Subtitles sub
	 */
	public Subtitles getSubtitles() {
		return this.sub;
	}

	/**
	 * Returns the player of type Player.
	 * @return Player player
	 */
	public Player getPlayer() {
		return this.player;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		if (controlsPanelMain == null)
			controlsPanelMain = mainPanel.getControlsPanel();
		if (effectsPanelMain == null)
			effectsPanelMain = mainPanel.getEffectsPanel();
		if (settingsPanelMain == null)
			settingsPanelMain = mainPanel.getSettingsPanel();
		player.getMediaPlayer().setAdjustVideo(true);
		

		/*
		 ************************************************************
		 ************************************************************
		 * Methods for Menu-Events
		 ************************************************************ 
		 ************************************************************
		 */

		if (e.getActionCommand() == "menu_open") {
			if(!this.isVideoLoaded) loadFile();
			else reloadFile();
		} 
		else if (e.getActionCommand() == "menu_exit") {
			System.out.println("UIListener:command=exit");
			//player.getMediaPlayer().stop();
			//mainPanel.getMediaListPlayer().stop();
			//mainPanel.getMediaListPlayer().release();
			//player.getMediaPlayer().release();
			System.exit(0);
		} 
		else if (e.getActionCommand() == "menu_save") {
			if(this.isVideoLoaded) {
				System.out.println("UIListener:command=saving");
				this.saveDialog = new PlayerSaveDialog(player,mainPanel, this.getVideoPath());			
				saving = true;
			}
			else {
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nPlease load a video first.", "Information", JOptionPane.INFORMATION_MESSAGE);
			}	 
		} 
		else if (e.getActionCommand() == "menu_about") {
			System.out.println("UIListener:command=about");
			this.aboutDialog = new PlayerAboutDialog(mainPanel);
		}
		else if (e.getActionCommand() == "menu_loadSub") {
			if(this.isVideoLoaded) {
				System.out.println("UIListener:command=load_subtitles");
				loadSubtitles();
				this.subtitlesLoaded = true;
			}
			else {
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nYou need to load a video to add Subtitles", "Information", JOptionPane.INFORMATION_MESSAGE);
			}	 
		}
		else if (e.getActionCommand() == "menu_createSub") {
			if(this.isVideoLoaded) {
				System.out.println("UIListener:command=create_subtitles");
				createSubtitles();
				this.settingsPanelMain.changeTabPane(1);
			}
			else {
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nYou need to load a video to add Subtitles.", "Information", JOptionPane.INFORMATION_MESSAGE);
			}	 
		}

		/*
		 ************************************************************
		 ************************************************************
		 * Methods for ControlPanel-Events
		 ************************************************************ 
		 ************************************************************
		 */

		else if (e.getActionCommand() == "reload") {
			loadFile();
		} 
		else if (e.getActionCommand() == "control_play") {
			if (mainPanel.isLoaded()) {
				if (controlsPanelMain.getStatus() == PlayerControlsPanel.STATUS_PLAY) {
					controlsPanelMain.setPaused();
					player.getMediaPlayer().pause();
					System.out.println("UIListener:command=pause");
				} else if (controlsPanelMain.getStatus() == PlayerControlsPanel.STATUS_PAUSE) {
					controlsPanelMain.setPlaying();
					
					//after saving
					if(saving) {
						player.getMediaPlayer().playMedia(filepath);
					}
					saving = false;
					player.getMediaPlayer().play();
					System.out.println("UIListener:command=play");
				}
			} else
				loadFile();
		} 
		else if (e.getActionCommand() == "control_backward") {
			System.out.println("UIListener:command=backward");
			skip(-10000);

		} 
		else if (e.getActionCommand() == "control_forward") {
			System.out.println("UIListener:command=forward");
			skip(10000);

		} 
		else if (e.getActionCommand() == "control_stop") {
			System.out.println("UIListener:command=stop");
			if(player.getMediaPlayer().isPlaying()) {
				player.getMediaPlayer().setPause(true);
				System.out.println("extra paused");
			}
			try {
				Thread.sleep(500);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			player.getMediaPlayer().setPosition(0.0f);
			
			controlsPanelMain.getTime().setText("00:00:00");
			mainPanel.updatePosition(0.0f);
			// change imageicon of play/pause to iconplay for startup
			controlsPanelMain.setPaused();
		} 
		else if (e.getActionCommand() == "control_mute") {
			if (controlsPanelMain.getVolStatus() == PlayerControlsPanel.STATUS_VOL) {
				controlsPanelMain.setMute();
				System.out.println("UIListener:command=mute");
			} else if (controlsPanelMain.getVolStatus() == PlayerControlsPanel.STATUS_MUTE) {
				controlsPanelMain.setTempVolume();
				System.out.println("UIListener:command=temp_volume");
			}
		}
		else if (e.getActionCommand() == "control_vol") {
			controlsPanelMain.setVolume();
			System.out.println("UIListener:command=full_volume");
		}

		/*
		 ************************************************************
		 ************************************************************
		 * Methods for EffectPanel-Events
		 ************************************************************ 
		 ************************************************************
		 */

		else if (e.getActionCommand() == "Greyscale" 
				|| e.getActionCommand() == "Original" 
				|| e.getActionCommand() == "Hue" 
				|| e.getActionCommand() == "Saturation" 
				|| e.getActionCommand() == "Invert"
				|| e.getActionCommand() == "Fisheye"){
			if(this.isVideoLoaded) {
				this.selectEffect(e.getActionCommand());
			}
			else {
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nTo add effects, please load a video first.", "Information", JOptionPane.INFORMATION_MESSAGE);
			}
			
		} 

		/*
		 ************************************************************
		 ************************************************************
		 * Methods for SettingsPanel-Events
		 ************************************************************ 
		 ************************************************************
		 */

		/*else if (e.getActionCommand() == "titel") {
			System.out.println("UIListener:command=Add_title");
			//float pos = player.getMediaPlayer().getPosition();
			//String title = settingsPanelMain.setTitle();

		}*/
		else if (e.getActionCommand() == "loadGraphics") {
			if(this.isVideoLoaded) {
				System.out.println("UIListener:command=load_graphics");
				loadGraphics();
			}
			else {
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nYou need to load a video to add Graphics.", "Information", JOptionPane.INFORMATION_MESSAGE);
			}	 
			
		} 
		else if (e.getActionCommand() == "graphics_add") {
			if(this.isVideoLoaded) {
				System.out.println("UIListener:command=add_graphics");
				
				this.images.setActiveImageHashMap(this.images.getActiveImageID());
				this.images.editImage((Long.parseLong(String.valueOf(this.settingsPanelMain.getImageStart()))), 
										(Long.parseLong(String.valueOf(this.settingsPanelMain.getImageDuration()))), 
										this.settingsPanelMain.getPosX(), 
										this.settingsPanelMain.getPosY(), 
										this.settingsPanelMain.getImageOpacity(), 
										this.settingsPanelMain.getImageScale());
			
				this.settingsPanelMain.updateImageList();
			}
			else {
				System.out.println("No Video loaded");
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No video-source found. \nYou need to load a video to add Graphics.", "Information", JOptionPane.INFORMATION_MESSAGE);
			}
		}
	}

	
	/*
	 ************************************************************
	 ************************************************************
	 * Methods used in this class
	 ************************************************************ 
	 ************************************************************
	 */

	private Effect activeEffect;
	private int oldEffectId;

	/**
	 * Applies the selected Effect to the video.
	 * @param name
	 */
	private void selectEffect(String name) {
		Effect effect = null;
		
		
		for (int i = 0; i < mainPanel.getEffectsPanel().getEffectFactory().getEffects().size(); i++) {
			effect = mainPanel.getEffectsPanel().getEffectFactory().getEffects().get(i);
			if (effect.getDisplayName() != name) {
				mainPanel.getEffectsPanel().getImagePanel().get(i).setVisible(true);
				mainPanel.getEffectsPanel().getSelectionList().get(i).setVisible(false);
			}
		}
		for (int i = 0; i < mainPanel.getEffectsPanel().getEffectFactory().getEffects().size(); i++) {
			effect = mainPanel.getEffectsPanel().getEffectFactory().getEffects().get(i);
			if (effect.getDisplayName() == name) {
				if(effect.configEffect()) { 
					System.out.println("OldEffectId: "+this.oldEffectId);
					this.setActiveEffect(effect);
					mainPanel.getEffectsPanel().getImagePanel().get(i).setVisible(false);
					mainPanel.getEffectsPanel().getSelectionList().get(i).setVisible(true);
					
					this.oldEffectId = i;
					
				} else {
					mainPanel.getEffectsPanel().getImagePanel().get(this.oldEffectId).setVisible(false);
					mainPanel.getEffectsPanel().getSelectionList().get(this.oldEffectId).setVisible(true);
				}
			}
		}
	}
	
	
	/**
	 * Loads the selected Graphic into the application.
	 */
	public void loadGraphics() {
		System.out.println("UIListener:command=load_graphic");
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setApproveButtonText("Laden");
		FileFilter defaultFilter = new FileNameExtensionFilter("Images", "jpg", "jpeg", "png", "gif");
		fileChooser.addChoosableFileFilter(defaultFilter);
		fileChooser.setFileFilter(defaultFilter);

		if (JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(mainPanel
				.getMainFrame())) {
			System.out.println(fileChooser.getSelectedFile().getAbsolutePath());
			filepath_img = fileChooser.getSelectedFile().getAbsolutePath();
			
			PlayerImage tmp = new PlayerImage(filepath_img);
			this.images.addImage(tmp);
			
			setLogoPath(tmp.getPath());
			this.settingsPanelMain.setPathInTextField(tmp.getPath());
			this.settingsPanelMain.setGraphicPreview(tmp.getThumb(70, 70));
			if(!this.settingsPanelMain.getPathInTextField().equals("")) {
				this.settingsPanelMain.getInsertBtn().setEnabled(true);
			}
			this.images.setActiveImageHashMap(this.images.getLastIndex()); //getLastIndex wird nur beim Laden eines neuen Bildes aufgerufen (Index!!)
			
		}
	}

	/**
	 * Loads video file for playback.
	 */
	public void loadFile() {
		System.out.println("UIListener:command=loading");
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setApproveButtonText("Abspielen");
		fileChooser.addChoosableFileFilter(SwingFileFilterFactory
				.newVideoFileFilter());
		FileFilter defaultFilter = SwingFileFilterFactory.newMediaFileFilter();
		fileChooser.addChoosableFileFilter(defaultFilter);
		fileChooser.setFileFilter(defaultFilter);

		if (JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(mainPanel
				.getMainFrame())) {;
				
				
			//show statusdialog
			statusDialog = new StatusDialog(mainPanel, 100);
			statusDialog.updateInfo("Loading video...");
			statusDialog.requestFocusInWindow();
			controlsPanelMain.setControlsDisabled(false);
			
			mainPanel.addImagePane();
			System.out.println(fileChooser.getSelectedFile().getAbsolutePath());
			filepath = fileChooser.getSelectedFile().getAbsolutePath();
			videoPath = filepath+"";
			player.getMediaPlayer().playMedia(
					fileChooser.getSelectedFile().getAbsolutePath());

			mainPanel.setLoaded(true);

			/* --- dirty fix for duration */
			player.getMediaPlayer().startMedia(
					fileChooser.getSelectedFile().getAbsolutePath());
			player.getMediaPlayer().setPosition(1f);
			System.out.println("length: " + player.getMediaPlayer().getTime());
			mainPanel.setMovieDuration(player.getMediaPlayer().getTime());
			player.getMediaPlayer().setPosition(0f);
			/* --- dirty fix end */
			
			mainPanel.setVideoToStart();
			this.isVideoLoaded = true;
			this.selectEffect("Original");
			
			//close status dialog
			statusDialog.dispose();
			controlsPanelMain.setControlsDisabled(true);
			
			//deleteSubtitles();
		}
		this.mainPanel.setMenuItemsEnabled(true);
		this.settingsPanelMain.setControlsDisabled(true);
	}
	
		

		/**
		 * Loads new video file for playback.
		 */
		public void reloadFile() {
			//check if video is already loaded
			if(isVideoLoaded) {
				if(player.getMediaPlayer().isPlaying()) {
					player.getMediaPlayer().pause();
				}
			}
			System.out.println("UIListener:command=reloading");
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setApproveButtonText("Play");
			fileChooser.addChoosableFileFilter(SwingFileFilterFactory
					.newVideoFileFilter());
			FileFilter defaultFilter = SwingFileFilterFactory.newMediaFileFilter();
			fileChooser.addChoosableFileFilter(defaultFilter);
			fileChooser.setFileFilter(defaultFilter);

			if (JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(mainPanel
					.getMainFrame())) {				
	
				//show statusdialog
				statusDialog = new StatusDialog(mainPanel, 100);
				statusDialog.updateInfo("Loading video...");
				statusDialog.requestFocusInWindow();
				controlsPanelMain.setControlsDisabled(false);
				
				filepath = fileChooser.getSelectedFile().getAbsolutePath();
				videoPath = filepath+"";
				this.mainPanel.addToMediaList(filepath);
				mainPanel.setLoaded(true);
				
				/*---dirty fix to update timeline*/
				this.mainPanel.setVideoToStart();
				//player.getMediaPlayer().stop();
				/*---dirty fix end */
				
				/* --- dirty fix for duration */
				player.getMediaPlayer().startMedia(filepath);
				player.getMediaPlayer().setPosition(1f);
				System.out.println("length: " + player.getMediaPlayer().getTime());
				mainPanel.setMovieDuration(player.getMediaPlayer().getTime());
				player.getMediaPlayer().setPosition(0f);
				/* --- dirty fix end */

				player.getMediaPlayer().pause();
				this.selectEffect("Original");
				
				//close status dialog
				statusDialog.dispose(); 
				controlsPanelMain.setControlsDisabled(true);
			}
			this.mainPanel.setMenuItemsEnabled(true);
			this.settingsPanelMain.setControlsDisabled(true);
	}

	/**
	 * Creates a new subtitle file.
	 */
	public void createSubtitles() {
		System.out.println("UIListener:command=create_subtitles");
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		//fileChooser.setApproveButtonText("Erstellen");
		FileFilter defaultFilter = SwingFileFilterFactory
				.newSubtitleFileFilter();
		fileChooser.addChoosableFileFilter(defaultFilter);
		fileChooser.setFileFilter(defaultFilter);

		if (JFileChooser.APPROVE_OPTION == fileChooser.showSaveDialog(mainPanel
				.getMainFrame())) {
			try {
				File f;
				if(fileChooser.getSelectedFile().getAbsolutePath().endsWith(".srt")) f = new File(fileChooser.getSelectedFile().getAbsolutePath());
				else f = new File(fileChooser.getSelectedFile().getAbsolutePath()+".srt");
				if(!f.exists()){
				  f.createNewFile();
				}
			} catch (IOException e) {
				
			}
			filepath = fileChooser.getSelectedFile().getAbsolutePath();
			player.getMediaPlayer().setSubTitleFile(filepath);
			sub = new Subtitles(filepath, this.player);

			this.settingsPanelMain.fillSubtitleList(sub.getSubtitles());
//			System.out.println("Subtitles: New Subtitle File created!");
//			System.out.println("Subtitles: "+sub.getSubtitles().size()+" Subtitles loaded");
			this.mainPanel.getTimeline().generateSubTitlePositions(sub.getSubtitles());
		}
	}
	
	/**
	 * Loads and opens an existing subtitle file.
	 */
	public void loadSubtitles() {
		System.out.println("UIListener:command=load_subtitles");
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setApproveButtonText("Load");
		FileFilter defaultFilter = SwingFileFilterFactory
				.newSubtitleFileFilter();
		fileChooser.addChoosableFileFilter(defaultFilter);
		fileChooser.setFileFilter(defaultFilter);

		if (JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(mainPanel
				.getMainFrame())) {
			filepath = fileChooser.getSelectedFile().getAbsolutePath();
			sub = new Subtitles(filepath, this.player);

			activateSubtitle(sub);
			
//			this.settingsPanelMain.fillSubtitleList(sub.getSubtitles());
//			System.out.println("Subtitles: "+sub.getSubtitles().size()+" Subtitles loaded");
//			this.mainPanel.getTimeline().generateSubTitlePositions(sub.getSubtitles());
//			player.getMediaPlayer().setSubTitleFile(sub.getFile());
			
			this.settingsPanelMain.changeTabPane(1);
		}
	}
	
	private void activateSubtitle(Subtitles sub){
		this.settingsPanelMain.fillSubtitleList(sub.getSubtitles());
		System.out.println("Subtitles: "+sub.getSubtitles().size()+" Subtitles loaded");
		this.mainPanel.getTimeline().generateSubTitlePositions(sub.getSubtitles());
		player.getMediaPlayer().setSubTitleFile(sub.getFile());
		System.out.println("loaded once");
		sub.saveAndReloadSubtitles();
		System.out.println("reloaded");
	}
	

	private void deleteSubtitles() {
		if(subtitlesLoaded){
			this.settingsPanelMain.emptySubtitleList();
			this.mainPanel.getTimeline().generateSubTitlePositions(new ArrayList<Subtitle>());
		}
	}

	/**
	 * Used to fast-forward video.
	 * @param skipTime
	 */
	private void skip(int skipTime) {
		if (player.getMediaPlayer().getLength() > 0)
			player.getMediaPlayer().skip(skipTime);
	}

	/**
	 * Converts input parameters (time) into milliseconds.
	 * @param h
	 * @param min
	 * @param sec
	 * @param milli
	 * @return
	 */
	public static int timeToMilli(int h, int min, int sec, int milli) {
		int minutes = (h * 60) + min;
		int seconds = (minutes * 60) + sec;
		int millisec = (seconds * 1000) + milli;

		return millisec;
	}

	/**
	 * Converts time from milliseconds to actual time(hh:mm:ss:milli).
	 * @param milli
	 * @return
	 */
	public static String milliToTime(long milli) {
		long millisec = milli % 1000;
		int seconds = ((int) Math.floor(milli / 1000) % 60);
		int minutes = ((int) Math.floor((int) Math.floor(milli / 1000) / 60) % 60);
		int hours = ((int) Math.floor((int) Math.floor((int) Math
				.floor(milli / 1000) / 60) / 60) % 60);

		return hours + ":" + minutes + ":" + seconds + "," + millisec;
	}

	/**
	 * Returns the path of the logo.
	 * @return
	 */
	public String getLogoPath() {
		return this.logopath;
	}
	
	
	/**
	 * Sets the path of the logo.
	 * @param logopath
	 */
	public void setLogoPath(String logopath) {
		this.logopath = logopath;
	}
	
	/**
	 * Returns the image ArrayList.
	 * @return
	 */
	public PlayerImages getImages() { return this.images; }
	
	/**
	 * Sets the saving status (true/false).
	 * @param status
	 */
	public void setSavingStatus(boolean status) {
		this.saving = true;
	}
	
	/**
	 * Returns filepath.
	 * Used in PlayerSaveDialog.
	 * @return
	 */
	public String getFilePath() {
		return this.filepath;
	}
	
	public String getVideoPath() {
		return this.videoPath;
	}

	/**
	 * Returns active Effect.
	 * @return
	 */
	public Effect getActiveEffect() {
		return activeEffect;
	}

	/**
	 * Sets the active Effect.
	 * @param activeEffect
	 */
	public void setActiveEffect(Effect activeEffect) {
		this.activeEffect = activeEffect;
	}
	

}
