package effects;

import java.awt.Color;
import java.awt.image.BufferedImage;

/**
 * Effect class.
 * @author Matthias Eigner
 *
 */
public class Effect implements IEffect {

	private String displayName;
	private BufferedImage thumb;
	
	/**
	 * Constructor of this class.
	 */
	public Effect() {
	}
	
	/**
	 * Sets Thumbnail.
	 * @param BufferedImage image
	 */
	public void setThumb(BufferedImage image){
		this.thumb = image;
	}
	
	/**
	 * Returns Thumbnail.
	 * @return BufferedImage thumb
	 */
	public BufferedImage getThumb() {
		return this.thumb;
	}

	/**
	 * Returns Name of Effect.
	 * @return String displayName
	 */
	public String getDisplayName() {
		return displayName;
	}

	/**
	 * Sets name of the Effect.
	 * @param String displayName
	 */
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}
	
	/* (non-Javadoc)
	 * @see effects.IEffect#applyEffect(int[])
	 */
	@Override
	public int[] applyEffect(int[] data) {
		return data;
	}
	
	/* (non-Javadoc)
	 * @see effects.IEffect#applyEffect(int[], double, double)
	 */
	@Override
	public int[] applyEffect(int[] data, double width, double height) {
		return data;
	}
	
	/* (non-Javadoc)
	 * @see effects.IEffect#configEffect()
	 */
	@Override
	public boolean configEffect() {return true;}
	
	float[] hsv = new float[3];
	/**
	 * Converts image from RGB to HSV.
	 * @param r
	 * @param g
	 * @param b
	 * @return
	 */
	protected float[] rgb2hsv(int r, int g, int b){
		Color.RGBtoHSB(r,g,b,this.hsv);
		return this.hsv;
	}
	
	/**
	 * Converts image from HSV to RGB.
	 * @param h
	 * @param s
	 * @param v
	 * @return
	 */
	protected int hsv2rgb(float h, float s, float v) {
		return Color.HSBtoRGB(h, s, v);
	}
	

	/* (non-Javadoc)
	 * @see effects.IEffect#applyEffectToBufferedImage(java.awt.image.BufferedImage)
	 */
	@Override
	public BufferedImage applyEffectToBufferedImage(BufferedImage img) {
		return img;
	}

}
