package effects;

import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.image.RescaleOp;

/**
 * Effect to display image in Black and white
 * this effect is quicker than using the saturation filter with saturation 0
 * @author Matthias Eigner 0625203
 *
 */
public class EffectBlackNWhite extends Effect implements IEffect {
	
	/**
	 * Constructor of the class.
	 */
	public EffectBlackNWhite() {
		super();
		this.setDisplayName("Greyscale");
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffect(int[])
	 */
	@Override
	public int[] applyEffect(int[] data) {
		int argb, r, g, b, grey;
		for (int i = 0; i < data.length; i++) {
			argb = data[i];
			b = (argb & 0xFF);
			g = 0;//((argb >> 8) & 0xFF);
			r = ((argb >> 16) & 0xFF);
			grey = (r+g+b+g) >> 2; //performance optimized -> not real grey!
			
			data[i] = (grey << 16) + (grey << 8) + grey;
		}
		return data;
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffectToBufferedImage(java.awt.image.BufferedImage)
	 */
	@Override
	public BufferedImage applyEffectToBufferedImage(BufferedImage img){
		ColorSpace cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
		ColorConvertOp op = new ColorConvertOp(cs, null);
		BufferedImage im = op.filter(img, null);

		// Brighten the image by 30%
		float scaleFactor = 4.0f;
		RescaleOp re = new RescaleOp(scaleFactor, 0, null);
		return re.filter(im, null);
	}
}
