package effects;

import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.LookupOp;
import java.awt.image.ShortLookupTable;

/**
 * Hue bestimmt den Farbton.
 * Man unterscheidet Farbempfindungen in Rot, Gelb und Grn.
 * 
 * @author Johann Houszka 0625523
 */
public class EffectInvert extends Effect implements IEffect {

	/**
	 * Constructor of this class.
	 */
	public EffectInvert() {
		super();
		this.setDisplayName("Invert");
	}

	/* (non-Javadoc)
	 * @see effects.Effect#applyEffect(int[])
	 */
	@Override
	public int[] applyEffect(int[] data) {
		int argb, r, g, b;

		for (int i = 0; i < data.length; i++) {
			argb = data[i];
			b = (argb & 0xFF);
			g = ((argb >> 8) & 0xFF);
			r = ((argb >> 16) & 0xFF);
			
			r = 255-r;
			g = 255-g;
			b = 255-b;
			
			data[i] = (r << 16) + (g << 8) + b;
		}
		
		return data;
	}
	
	/**
	 * Table used for the effect.
	 */
	private static final short[] invertTable;
	static {
		invertTable = new short[256];
		for (int i = 0; i < 256; i++) {
			invertTable[i] = (short) (255 - i);
		}
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffectToBufferedImage(java.awt.image.BufferedImage)
	 */
	@Override
	public BufferedImage applyEffectToBufferedImage(BufferedImage img) {
		final int w = img.getWidth();
		final int h = img.getHeight();
		final BufferedImage dst = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_RGB);

		final BufferedImageOp invertOp = new LookupOp(new ShortLookupTable(0,
				invertTable), null);
		return invertOp.filter(img, dst);
	}
}