package effects;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;

/**
 * Saturation Effect for Project TitleEditor 
 * @author Matthias Eigner 0625203
 */
public class EffectSaturation extends Effect implements IEffect {

	private float saturation;
	
	/**
	 * Constructor of this class.
	 */
	public EffectSaturation() {
		super();
		this.setDisplayName("Saturation");
	}

	/* (non-Javadoc)
	 * @see effects.Effect#applyEffect(int[])
	 */
	@Override
	public int[] applyEffect(int[] data) {
		int argb, r, g, b;
		double p;
		
		float Pr = .299f;
		float Pg = .587f;
		float Pb = .114f;
		
		for (int i = 0; i < data.length; i++) {
			argb = data[i];
			b = (argb & 0xFF);
			g = ((argb >> 8) & 0xFF);
			r = ((argb >> 16) & 0xFF);
			
			p = Math.sqrt(r*r*Pr + g*g*Pg + b*b*Pb);
			
			r = (int)(p+(r-p)*this.saturation*2);
			g = (int)(p+(g-p)*this.saturation*2);
			b = (int)(p+(b-p)*this.saturation*2);
			
			if(r > 255) r = 255;
			if(g > 255) g = 255;
			if(b > 255) b = 255;

			if(r < 0) r = 0;
			if(g < 0) g = 0;
			if(b < 0) b = 0;

			data[i] = (r << 16) + (g << 8) + b;
		}
		return data;
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#configEffect()
	 */
	@Override
	public boolean configEffect() {
		return this.createConfigInterface();
	}
	
	/**
	 * @wbp.parser.entryPoint
	 */
	private boolean createConfigInterface() {
		JPanel optionsPanel = new JPanel();
		optionsPanel.setLayout(null);
		optionsPanel.setPreferredSize(new Dimension(321, 42));
		
		JSlider slider = new JSlider();
		slider.setMaximum(1000);
		slider.setValue((int)(this.saturation * 1000));
		slider.setMinorTickSpacing(1);
		slider.setBounds(93, 6, 222, 29);
		optionsPanel.add(slider);
		
		JLabel lblSaturation = new JLabel("Saturation:");
		lblSaturation.setBounds(6, 10, 75, 16);
		optionsPanel.add(lblSaturation);

		int response = JOptionPane.showConfirmDialog(null, optionsPanel,
				"Options for Saturation Effect", JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.PLAIN_MESSAGE);
		
		if(response == JOptionPane.OK_OPTION) {
			this.saturation = (float)slider.getValue()/1000.0f;
			return true;
		} else if(response == JOptionPane.CANCEL_OPTION) {
			return false;
		}
		
		return false;
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffectToBufferedImage(java.awt.image.BufferedImage)
	 */
	@Override
	public BufferedImage applyEffectToBufferedImage(BufferedImage img){
		/**
	     * Adjusts the saturation of the component image and draws the resulting
	     * image using the component's graphics.
	     */
        int width = img.getWidth();
        int height = img.getHeight();
        Graphics graphics = img.getGraphics();
        
        int[] buffer = new int[width * height];

        WritableRaster raster = img.getRaster();
        raster.getDataElements(0, 0, width, height, buffer);

        float[] hsb = new float[3];

        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                int srcRGB = buffer[i * width + j];

                // Adjust color saturation
                Color.RGBtoHSB((srcRGB >> 16) & 0xff,
                    (srcRGB >> 8) & 0xff, srcRGB & 0xff, hsb);
                int dstRGB = Color.HSBtoRGB(hsb[0],
                    Math.min(Math.max(hsb[1] * 3, 0f), 1f), hsb[2]);

                // Preserve the source alpha channel
                dstRGB = (srcRGB & 0xff000000) | (dstRGB & 0xffffff);

                buffer[i * width + j] = dstRGB;
            }
        }
        raster.setDataElements(0, 0, width, height, buffer);
        graphics.drawImage(img, 0, 0, null);
        
        return img;
	}
}
