package model;

import java.awt.Dimension;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;

import controller.FastImageScaler;

/**
 * Defines a PlayerImage. 
 * Consists of Getter/Setter Methodes to define values of the Image.
 * @author Johann Houszka, Matthias Eigner
 *
 */
public class PlayerImage {
	private BufferedImage image;
	private String path;
	private String name;
	private long start;
	private long duration;
	private int opacity;
	private int scale;
	private int pos_x;
	private int pos_y;
	private int id;
	
	/**
	 * Create a new PlayerImage by filepath
	 */
	public PlayerImage(String path) {
		this.path = path;
		ImageIcon i = new ImageIcon(path);
		this.image = new BufferedImage(i.getIconWidth(), i.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
		this.image.getGraphics().drawImage(i.getImage(), 0, 0, i.getImageObserver());
		String[] filename = path.split("/");
		this.name = filename[filename.length-1];
	}

	/**
	 * Get Image of type BufferedImage
	 * @return BufferedImage image
	 */
	public BufferedImage getImage() {
		return this.image;
	}
	
	
	/**
	 * Returns scaled Thumb.
	 * @param width
	 * @param height
	 * @return ImageIcon
	 */
	public ImageIcon getThumb(int width, int height) {
		if(width == -1) width = this.image.getWidth();
		if(height == -1) height = this.image.getHeight();
		
		FastImageScaler scaler = new FastImageScaler();
		return new ImageIcon(scaler.scaleImage(this.image, new Dimension(width,height)));
	}

	
	/**
	 * Returns starttime.
	 * @return int start
	 */
	public long getStart() {
		return start;
	}

	/**
	 * Sets starttime.
	 * @param start
	 */
	public void setStart(long start) {
		this.start = start;
	}

	/**
	 * Returns duration.
	 * @return int duration
	 */
	public long getDuration() {
		return duration;
	}

	/**
	 * Sets Duration of graphic to be shown.
	 * @param duration
	 */
	public void setDuration(long duration) {
		this.duration = duration;
	}

	/**
	 * Returns opacity.
	 * @return int opacity.
	 */
	public int getOpacity() {
		return opacity;
	}

	/**
	 * Sets opacity of graphic.
	 * @param opacity
	 */
	public void setOpacity(int opacity) {
		this.opacity = opacity;
	}

	/**
	 * Returns scale-factor.
	 * @return int scale
	 */
	public int getScale() {
		return scale;
	}

	/**
	 * Sets scale-factor.
	 * @param scale
	 */
	public void setScale(int scale) {
		this.scale = scale;
	}

	/**
	 * Returns posx.
	 * @return int posx
	 */
	public int getPos_x() {
		return pos_x;
	}

	/**
	 * Sets posx.
	 * @param pos_x
	 */
	public void setPos_x(int pos_x) {
		this.pos_x = pos_x;
	}

	/**
	 * Returns posy.
	 * @return int posy
	 */
	public int getPos_y() {
		return pos_y;
	}

	/**
	 * Sets posy.
	 * @param pos_y
	 */
	public void setPos_y(int pos_y) {
		this.pos_y = pos_y;
	}
	
	/**
	 * Returns path to image-file.
	 * @return String path
	 */
	public String getPath() {
		return this.path;
	}
	
	/**
	 * Returns name of file.
	 * @return String name
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * Sets ID of graphic.
	 * @param id
	 */
	public void setID(Integer id) {
		this.id = id;
	}
	
	/**
	 * Returns ID of graphic.
	 * @return id
	 */
	public Integer getID() {
		return this.id;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override 
	public boolean equals(Object o_){
		if(o_ instanceof PlayerImage){
			PlayerImage o = (PlayerImage)o_;
			return this.image.equals(o.getImage()) && 
					this.start == o.getStart() &&
					this.duration == o.getDuration() &&
					this.pos_x == o.getPos_x() &&
					this.pos_y == o.getPos_y() &&
					this.scale == o.getScale();
		} else return false;
	}
}
