package model;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;

/**
 * Class for handling Images.
 * @author Matthias Eigner, Johann Houszka
 *
 */
public class PlayerImages {
	private ArrayList<PlayerImage> images = new ArrayList<PlayerImage>();
	private PlayerImage activeImage;
	private int id;
	private HashMap<Integer, PlayerImage> hashmap = new HashMap<Integer,PlayerImage>();
	private int i = 0; 
	private int index = 0; 
	
	
	/**
	 * Returns an PlayerImage from the ArrayList<PlayerImage>
	 * @param img
	 * @return
	 */
	public PlayerImage getImage(PlayerImage img) {
		return images.get(images.indexOf(img));
	}
	
	/**
	 * Adds image to hashmap and to ArrayList<PlayerImage>.
	 * @param img
	 */
	public void addImage(PlayerImage img){
		img.setID(getCurrentIndex());
		this.setActiveImageID(img.getID());
		//System.out.println("ActiveImage ID: " + getActiveImageID());
		//System.out.println("CurrentIndex: " + getCurrentIndex());
		this.hashmap.put(getCurrentIndex(), img);
		//System.out.println("AddImage::HashMapSize: " + this.hashmap.size());

		this.images.add(img);
	}
	
	/**
	 * Returns currently selected image.
	 * @return
	 */
	public PlayerImage getActiveImageHashMap() {
		return this.activeImage;
	}
	
	/**
	 * Sets currently selected image.
	 * @param id
	 */
	public void setActiveImageHashMap(int id) {
		this.activeImage = this.hashmap.get(id);
	}
	
	/**
	 * Returns ArrayList with all loaded images.
	 * @return
	 */
	public ArrayList<PlayerImage> getImages() {
		return this.images;
	}
	
	/**
	 * Sets the ID of the currently active (selected) image.
	 * @param id
	 */
	public void setActiveImageID(int id) {
		this.id = id;	
	}
	
	/**
	 * Returns ID of the currently selected image.
	 * @return int id
	 */
	public int getActiveImageID() {
		return this.id;
	}
	
	/**
	 * Returns a RenderedImage with all currently loaded images (all images in hashmap).
	 * @param width
	 * @param height
	 * @return BufferedImage tmp
	 */
	public Image getRenderedImage(double width, double height) {
		BufferedImage tmp = new BufferedImage((int)(width), (int)(height), BufferedImage.TYPE_INT_ARGB);
		AlphaComposite ac;
		for(int i=0; i< this.hashmap.size(); i++){ 
			PlayerImage i_ = hashmap.get(i);

			ac = java.awt.AlphaComposite.getInstance(AlphaComposite.CLEAR, (((float)i_.getOpacity())/100));
			((Graphics2D) tmp.getGraphics()).setComposite(ac);
			tmp.getGraphics().drawImage(i_.getImage(), 
										i_.getPos_x(), i_.getPos_y(), 
										i_.getImage().getWidth()*i_.getScale()/100, 
										i_.getImage().getHeight()*i_.getScale()/100, 
										null);
		}
		return tmp;
	}
	
	/**
	 * Returns the ImageListContent.
	 * Content is displayed in list.
	 * @return DefaultListModel list
	 */
	public DefaultListModel getImageListContent() {
		DefaultListModel list = new DefaultListModel();
		
		Iterator<Integer> iterator =  hashmap.keySet().iterator();
		//System.out.println("Iterator Size: " + hashmap.keySet().size());
		list.add(list.getSize(),new Object[]{null,"-- New Graphic --"});
		while(iterator.hasNext()){
		    int key = iterator.next();
		    //System.out.println("Iterator Key: " + key);
		    PlayerImage im = hashmap.get(key);
			//System.out.println("Method: getImageListContent: " + im.getName() + "::" + im.getID());
			list.add(list.getSize(), 
					new Object[]{im.getThumb(40, 40), im.getName()});
		}
		return list;
	}
	
	/**
	 * Returns the ImageListStore.
	 * Used to load current values into the fields.
	 * @return ArrayList(ImageStore)
	 */
	public ArrayList<ImageStore> getImageListStore() {	
		ArrayList<ImageStore> list = new ArrayList<ImageStore>();
		
		Iterator<Integer> iterator =  hashmap.keySet().iterator();
		//System.out.println("Iterator Size: " + hashmap.keySet().size());
		
		//first item is the "New Graphic" element
		if(list.size() == 0) {
			ImageStore store = new ImageStore();
			store.setIndex(0);
			store.setImageIcon(null);
			store.setImage(null);
			list.add(store);
		}
		
		while(iterator.hasNext()){
			ImageStore store = new ImageStore();
		    int key = iterator.next();
		    store.setIndex(key);
		    PlayerImage im = hashmap.get(key);
		    store.setImage(im);
		    store.setImageIcon(im.getThumb(-1, -1));
		    store.setFilepath(im.getPath());
		    list.add(store);
			//System.out.println("ImageListStore Size: " + list.size());
		}
		return list;
	}
	
	/**
	 * Edits activeImage in images list
	 * @param start
	 * @param duration
	 * @param x
	 * @param y
	 * @param opacity
	 * @param scale
	 * @return
	 */
	public void editImage(long start, long duration, int x, int y, int opacity, int scale){
		if(this.hashmap.containsValue(activeImage)){
			PlayerImage i_ = this.hashmap.get(getKeyByValue(this.hashmap, activeImage));
			i_.setStart(start);
			i_.setDuration(duration);
			i_.setPos_x(x);
			i_.setPos_y(y);
			i_.setOpacity(opacity);
			i_.setScale(scale);
			
			System.out.println("EditImage: Put edited image into Hashmap on index " + i_.getID());
		} else {
			System.out.println("EditImage: Put new image into Hashmap with index " + getCurrentIndex());
			this.hashmap.put(getCurrentIndex(),getActiveImageHashMap());
		}
	}
	
	/**
	 * Helper Method to get Key from Value in HashMap
	 */
	public static <T, E> T getKeyByValue(Map<T, E> map, E value) {
	    for (Entry<T, E> entry : map.entrySet()) {
	        if (value.equals(entry.getValue())) {
	            return entry.getKey();
	        }
	    }
	    return null;
	}
	
	/**
	 * Returns a merged BufferedImage of all Images 
	 * visible at given Position
	 * @param position
	 * @param width
	 * @param height
	 * @return
	 */
	public BufferedImage getImageAtPosition(long position, int width, int height){
		BufferedImage tmp = new BufferedImage((int)(width), (int)(height), BufferedImage.TYPE_INT_ARGB);
		PlayerImage i_;

		for(int i=0; i< this.hashmap.size(); i++){ 
			i_ = this.hashmap.get(i);
			if(i_.getStart() <= position && i_.getStart()+i_.getDuration() >= position){
				tmp.getGraphics().drawImage(i_.getImage(), 
						i_.getPos_x()-(i_.getImage().getWidth()/2), 
						i_.getPos_y()-(i_.getImage().getHeight()/2), 
						i_.getImage().getWidth()*i_.getScale()/100, 
						i_.getImage().getHeight()*i_.getScale()/100, 
						null);
			}
		}
		return tmp;
	}
	
	/**
	 * Removes selected image from ArrayList and Hashmap.
	 * @param image
	 */
	public void deleteImage(PlayerImage image) {
		if(this.images.contains(image)) {
			this.images.remove(image);
		}
		if(this.hashmap.containsValue(image)) {
			this.hashmap.remove(image.getID());
		}
		
	}
	
	/**
	 * Returns last index 
	 * used to add index to new images
	 */
	public int getLastIndex() {
		System.out.println("Last used Index:" + i);
		updateNewIndex(); //this is always the new index (first unused number)
		
		return getCurrentIndex(); //return current index (last used number)
	}
	
	/**
	 * Updates index of images in list.
	 */
	private void updateNewIndex() {
		this.i = i+1;
		setCurrentIndex(i);
	}
	
	
	/**
	 * Returns the new index (old index +1).
	 * @return
	 */
	public int getNewIndex() {
		return this.i; 
	}
	
	/**
	 * Return current index.
	 * @return
	 */
	public int getCurrentIndex() {
		return this.index;
	}
	
	
	/**
	 * Set the current index.
	 * @param index
	 */
	public void setCurrentIndex(int index) {
		this.index = index;
	}
	
	
	/**
	 * Used to save the index in combination with an ImageIcon
	 * update: now also the whole PlayerImage is stored here (01.04.2013)
	 * @author Johann Houszka
	 *
	 */
	public class ImageStore {
		
		private ImageIcon icon;
		private int index;
		private PlayerImage image;
		private String filepath;
		
		/**
		 * Sets ImageIcon for a PlayerImage.
		 * @param icon
		 */
		protected void setImageIcon(ImageIcon icon) {
			this.icon = icon;
		}
		
		/**
		 * Returns ImageIcon for a PlayerImage.
		 * @return
		 */
		public ImageIcon getIcon() {
			return this.icon;
		}
		
		/**
		 * Set Index for a PlayerImage.
		 * @param index
		 */
		protected void setIndex(int index) {
			this.index = index;
		}
		
		/**
		 * Returns Index for a PlayerImage.
		 * @return
		 */
		public int getIndex() {
			return this.index;
		}
		
		/**
		 * Sets whole PlayerImage.
		 * @param image
		 */
		protected void setImage(PlayerImage image) {
			this.image = image;
		}
		
		/**
		 * Returns whole PlayerImage.
		 * @return
		 */
		public PlayerImage getImage() {
			return this.image;
		}
		
		/**
		 * Sets Filepath for a PlayerImage.
		 * @param filepath
		 */
		protected void setFilepath(String filepath) {
			this.filepath = filepath;
		}
		
		/**
		 * Returns Filepath for a PlayerImage.
		 * @return
		 */
		public String getFilepath() {
			return this.filepath;
		}
		
	}
}
