package model;

import controller.PlayerUIListener;

/**
 * Class for Subtitle.
 * @author Matthias Eigner
 *
 */
public class Subtitle {
	private String value;
	private long start;
	private long end;
	
	/**
	 * Constructor of this class.
	 */
	public Subtitle() {
		this.value = "";
		this.start = 0;
		this.end = 0;
	}
	
	/**
	 * Constructor with parameters for this class.
	 * @param start
	 * @param end
	 * @param value
	 */
	public Subtitle(long start, long end, String value){
		this.value = value;
		this.start = start;
		this.end = end;
		System.out.println(value+"-");
	}

	/**
	 * Add Line in subtitle-textfile.
	 * @param line
	 */
	public void addLine(String line) {
		if(this.value == "") this.value = line;
		else this.value += "\n"+line;
		System.out.println(this.value);
	}
	
	/**
	 * Set duration to given subtitle.
	 * @param duration
	 */
	public void setDuration(long duration){
		if(duration > 0){
			this.end = this.start + duration;
		}
	}
	
	/**
	 * Set starttime to given subtitle.
	 * @param start
	 */
	public void setStartLeaveDuration(long start){
		long duration = this.end - this.start;
		this.start = start;
		this.setDuration(duration);
	}
	
	/**
	 * Return String with starttime and duration to add to subtitle-textfile.
	 * @param id
	 * @return String timeString.
	 */
	public String toFile(int id) {
		return "\n"+this.toFileFirst(id);
	}
	
	public String toFileFirst(int id) {
		return id+"\n" + PlayerUIListener.milliToTime(this.getStart()) + " --> " + PlayerUIListener.milliToTime(this.getEnd()) + "\n" + this.getValue() + "\n";
	}
	
	/*
	 ************************************************************
	 ************************************************************
	 * Setter/Getter Methods for this class.
	 ************************************************************ 
	 ************************************************************
	 */
	
	/**
	 * Return value for given subtitle.
	 * @return
	 */
	public String getValue() {
		return value;
	}

	/**
	 * Set value for given subtitle.
	 * @param value
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 * Return starttime for given subttitle.
	 * @return
	 */
	public long getStart() {
		return start;
	}

	/**
	 * Set starttime for given subtitle.
	 * @param start
	 */
	public void setStart(long start) {
		this.start = start;
	}

	/**
	 * Return enddtime for given subtitle.
	 * @return
	 */
	public long getEnd() {
		return end;
	}

	/**
	 * Set endtime for subtitle.
	 * @param end
	 */
	public void setEnd(long end) {
		this.end = end;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override 
	public boolean equals(Object o_){
		if(o_ instanceof Subtitle){
			Subtitle o = (Subtitle)o_;
			return this.start == o.start && this.end == o.end && this.value == o.value;
		} else return false;
	}
}
