package model;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import player.Player;

/**
 * Class for handling Title data
 * @author Matthias Eigner
 *
 */
public class Titles {
	
	private ArrayList<Title> titles;
	private Player player;
	
	/**
	 * Constructor of this class.
	 * @param player
	 */
	public Titles(Player player){
		this.player = player;
		this.titles = new ArrayList<Title>();
	}
	
	public ArrayList<Title> getTitles() { return this.titles; }
	
	/**
	 * Edits title and updates values.
	 * @param t
	 * @param content
	 * @param font
	 * @param color
	 * @param size
	 * @param start
	 * @param duration
	 * @param x
	 * @param y
	 * @return
	 */
	public boolean editTitle(Title t, String content, int font, String color, int size, long start, long duration, int x, int y){
		if(this.titles.contains(t)){
			Title t_ = this.titles.get(this.titles.indexOf(t));
			t_.setContent(content);
			t_.setColor(color);
			t_.setFont(font);
			t_.setSize(size);
			t_.setStart(start);
			t_.setDuration(duration);
			t_.setX(x);
			t_.setY(y);
			return true;
		} else return false;
	}
	
	/**
	 * Insert Title into List.
	 * @param t
	 * @return
	 */
	public boolean insertTitle(Title t){
		this.titles.add(t);
		return true;
	}
	
	/**
	 * Deletes Title from List.
	 * @param t
	 */
	public void deleteTitle(Title t) {
		if(this.titles.contains(t)) {
			this.titles.remove(t);
		}
	}
	
	private BufferedImage tmp;
	/**
	 * Returnes a merged BufferedImage of all Titles 
	 * visible at given Position
	 * @param position
	 * @param width
	 * @param height
	 * @return
	 */
	public BufferedImage getImageAtPosition(long position, int width, int height){
		tmp = new BufferedImage((int)(width), (int)(height), BufferedImage.TYPE_INT_ARGB);
		Title t_;
		Graphics2D g2d;
		for(int i=0; i< this.titles.size(); i++){
			t_ = this.titles.get(i);
			if(t_.getStart() <= position && t_.getStart()+t_.getDuration() >= position){
				g2d = tmp.createGraphics();
				Color c = Titles.hex2Rgb(t_.getColor());
				g2d.setColor(c);
//				System.out.println(this.getFontFromId(t_.getFont(), t_.getSize()).getFontName());
				g2d.setFont(this.getFontFromId(t_.getFont(), t_.getSize()));
			    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			    g2d.drawString(t_.getContent(), t_.getX(), t_.getY());
			}
		}
		return tmp;
	}
		
	private Font[] Fonts;
	/**
	 * Get the font from the ID.
	 * @param id
	 * @param size
	 * @return
	 */
	private Font getFontFromId(int id, int size) {
		if(this.Fonts == null) {
			String[] f = this.player.getFonts();
			this.Fonts = new Font[f.length];
			for(int i=0; i<f.length;i++){
				this.Fonts[i] = new Font(f[i], Font.PLAIN, 20);
			}
		}
		//System.out.println(this.Fonts[id].getFamily());
		if(id < this.Fonts.length){
			return this.Fonts[id].deriveFont(size+0.0f);
		} else return null;
	}
	
	/**
	 * Converts HEX to RGB.
	 * @param colorStr
	 * @return
	 */
	private static Color hex2Rgb(String colorStr) {
		colorStr = colorStr.replace("#","");
		if(colorStr.length() == 6){
			return new Color(
	            Integer.valueOf( colorStr.substring( 0, 2 ), 16 ),
	            Integer.valueOf( colorStr.substring( 2, 4 ), 16 ),
	            Integer.valueOf( colorStr.substring( 4, 6 ), 16 ) );
		} else return new Color(255,255,255);
	}
}
