package player;

import java.awt.GraphicsEnvironment;
import java.util.ArrayList;
import java.util.List;

import uk.co.caprica.vlcj.logger.Logger;
import uk.co.caprica.vlcj.player.AudioOutput;
import uk.co.caprica.vlcj.player.MediaPlayerFactory;
import uk.co.caprica.vlcj.player.direct.DirectMediaPlayer;
import view.PlayerMainPanel;

import com.sun.jna.NativeLibrary;

import controller.Settings;

/**
 * Player Class. Instantiates the UI.
 * @author Johann Houszka, Matthias Eigner
 *
 */
public class Player {
	private String sourceurl;
	PlayerMainPanel mainPanel;
    private MediaPlayerFactory mediaPlayerFactory;
    private DirectMediaPlayer mediaPlayer;
    private Settings settings;
	
	/**
	 * Constructor of this class.
	 * Used to add vlc-Arguments to the application.
	 * @param vlc_path
	 */
	public Player(){
		this.settings = new Settings();
		sourceurl = this.settings.getLibPath("vlc");
		//sourceurl = "\"" + sourceurl + "\"";
		//sourceurl = this.quoteWhitespaces(sourceurl);
		System.out.println("VLC-Pfad: " + sourceurl);
		if(sourceurl != null) {
			this.initFonts();
        	NativeLibrary.addSearchPath("libvlc", sourceurl);
        	System.setProperty("jna.library.path", sourceurl);
        	
        	List<String> vlcArgs = new ArrayList<String>();
            vlcArgs.add("--no-plugins-cache");
            vlcArgs.add("--no-video-title-show");
            //vlcArgs.add("--no-sub-autodetect-file");
            vlcArgs.add("--no-snapshot-preview");
            vlcArgs.add("--quiet");
            vlcArgs.add("--quiet-synchro");
            vlcArgs.add("--intf");
            vlcArgs.add("--subsdec-align=1");
            //vlcArgs.add("--blur-factor=110");
            vlcArgs.add("dummy");
            
            Logger.debug("vlcArgs={}", vlcArgs);

            mediaPlayerFactory = new MediaPlayerFactory(vlcArgs.toArray(new String[vlcArgs.size()]));
            mediaPlayerFactory.setUserAgent("Titel Editor");
            List<AudioOutput> audioOutputs = mediaPlayerFactory.getAudioOutputs();
            Logger.debug("audioOutputs={}", audioOutputs);

        	mainPanel = new PlayerMainPanel(this);
		}
	}
	

	public String quoteWhitespaces(String path) {
		return path.replaceAll(" ", "\" \"");
	}
	
	/**
	 * Returns the MediaPlayerFactory.
	 * @return mediaPlayerFactory
	 */
	public MediaPlayerFactory getMediaPlayerFactory() {
		return mediaPlayerFactory;
	}

	/**
	 * Sets the MediaPlayerFactory.
	 * @param MediaPlayerFactory mediaPlayerFactory
	 */
	public void setMediaPlayerFactory(MediaPlayerFactory mediaPlayerFactory) {
		this.mediaPlayerFactory = mediaPlayerFactory;
	}

	/**
	 * Returns the MediaPlayer
	 * @return mediaPlayer
	 */
	public DirectMediaPlayer getMediaPlayer() {
		return mediaPlayer;
	}

	/**
	 * Set the MediaPlayer.
	 * @param DirectMediaPlayer mediaPlayer
	 */
	public void setMediaPlayer(DirectMediaPlayer mediaPlayer) {
		this.mediaPlayer = mediaPlayer;
	}
	
	private String[] fonts;
	
	/**
	 * Initializes fonts. 
	 * All Fonts installed on the current system can be used.
	 */
	private void initFonts() {
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		this.fonts = ge.getAvailableFontFamilyNames();
	}
	
	/**
	 * Set Fonts from Array.
	 * @param String[] fonts
	 */
	public void setFonts(String[] fonts) {
		this.fonts = fonts;
	}
	
	/**
	 * Return Fonts in StringArray.
	 * @return fonts
	 */
	public String[] getFonts() {
		return fonts;
	}
	
	/**
	 * returnes settings object
	 * @return
	 */
	public Settings getSettings() {
		return this.settings;
	}
}
