package save;

import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBufferInt;
import java.io.File;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.concurrent.atomic.AtomicReference;

import com.xuggle.ferry.JNIReference;
import com.xuggle.mediatool.IMediaReader;
import com.xuggle.mediatool.IMediaWriter;
import com.xuggle.mediatool.MediaToolAdapter;
import com.xuggle.mediatool.ToolFactory;
import com.xuggle.mediatool.event.AudioSamplesEvent;
import com.xuggle.mediatool.event.IAddStreamEvent;
import com.xuggle.mediatool.event.IAudioSamplesEvent;
import com.xuggle.mediatool.event.IVideoPictureEvent;
import com.xuggle.mediatool.event.VideoPictureEvent;
import com.xuggle.xuggler.IAudioResampler;
import com.xuggle.xuggler.IAudioSamples;
import com.xuggle.xuggler.ICodec;
import com.xuggle.xuggler.IPixelFormat;
import com.xuggle.xuggler.IStreamCoder;
import com.xuggle.xuggler.IVideoPicture;
import com.xuggle.xuggler.IVideoResampler;

public class ConvertVideo extends MediaToolAdapter implements Runnable {
	 private int VIDEO_WIDTH = 712;
	 private int VIDEO_HEIGHT = 429;
	 
	 private IMediaWriter writer;
	 private IMediaReader reader;
	 private File outputFile;
		 
	 public ConvertVideo(File inputFile, File outputFile) {
		 this.outputFile = outputFile;
		 System.out.println("OutputFile: " + outputFile);
		 System.out.println("InputFile: " + inputFile);
		 reader = ToolFactory.makeReader(inputFile.getAbsolutePath());
		 reader.addListener(this);
	 }
	 
	 private IVideoResampler videoResampler = null;
	 private IAudioResampler audioResampler = null;
	 
	 @Override
	 public void onAddStream(IAddStreamEvent event) {
		 int streamIndex = event.getStreamIndex();
		 IStreamCoder streamCoder = event.getSource().getContainer().getStream(streamIndex).getStreamCoder();
		 if (streamCoder.getCodecType() == ICodec.Type.CODEC_TYPE_AUDIO) {
		 writer.addAudioStream(streamIndex, streamIndex, 2, 44100);
		 } else if (streamCoder.getCodecType() == ICodec.Type.CODEC_TYPE_VIDEO) {
		 streamCoder.setWidth(VIDEO_WIDTH);
		 streamCoder.setHeight(VIDEO_HEIGHT);
		 writer.addVideoStream(streamIndex, streamIndex, VIDEO_WIDTH, VIDEO_HEIGHT);
		 }
		 super.onAddStream(event);
	 }
	 
	 public void onVideoPicture(BufferedImage image, long timestamp) {//(IVideoPictureEvent event) {
		 System.out.println("Method: onVideoPicture");
		 IVideoPicture pic = toPicture(image, timestamp);
		 //IVideoPicture pic = event.getPicture();
		 if (videoResampler == null) {
		 videoResampler = IVideoResampler.make(VIDEO_WIDTH, VIDEO_HEIGHT, pic.getPixelType(), pic.getWidth(), pic.getHeight(), pic.getPixelType());
		 }
		 IVideoPicture out = IVideoPicture.make(pic.getPixelType(), VIDEO_WIDTH, VIDEO_HEIGHT);
		 videoResampler.resample(out, pic);
		 
		 //IVideoPictureEvent asc = new VideoPictureEvent(event.getSource(), out, event.getStreamIndex());
		 //super.onVideoPicture(asc);
		 out.delete();
	 }
	 
	 @Override
	 public void onAudioSamples(IAudioSamplesEvent event) {
		 IAudioSamples samples = event.getAudioSamples();
		 if (audioResampler == null) {
		 audioResampler = IAudioResampler.make(2, samples.getChannels(), 44100, samples.getSampleRate());
		 }
		 if (event.getAudioSamples().getNumSamples() > 0) {
		 IAudioSamples out = IAudioSamples.make(samples.getNumSamples(), samples.getChannels());
		 audioResampler.resample(out, samples, samples.getNumSamples());
		 
		 AudioSamplesEvent asc = new AudioSamplesEvent(event.getSource(), out, event.getStreamIndex());
		 super.onAudioSamples(asc);
		 out.delete();
		 }
	 }
	 
	 @Override
	 public void run() {
		 writer = ToolFactory.makeWriter(outputFile.getAbsolutePath(), reader);
		 System.out.println("Method: " + "ConvertVideo.run()");
		 this.addListener(writer);
		 while (reader.readPacket() == null) {
		 }
	 }
	 
	 public static BufferedImage convertToType(BufferedImage sourceImage, int targetType)
	 {
		 BufferedImage image;

		 // if the source image is already the target type, return the source image
		 if (sourceImage.getType() == targetType)
			 image = sourceImage;
		 // otherwise create a new image of the target type and draw the new image 
		 else
		 {
			 image = new BufferedImage(sourceImage.getWidth(), sourceImage.getHeight(), targetType);
			 image.getGraphics().drawImage(sourceImage, 0, 0, null);
		 }  
		 return image;
	 }
	 
	 public IVideoPicture toPicture(BufferedImage image, long timestamp)
	  {
	    // validate the image

	    //validateImage(image);

	    // get the image byte buffer buffer

	    DataBuffer imageBuffer = image.getRaster().getDataBuffer();
	    byte[] imageBytes = null;
	    int[] imageInts = null;

	    // handle byte buffer case

	    if (imageBuffer instanceof DataBufferByte)
	    {
	      imageBytes = ((DataBufferByte)imageBuffer).getData();
	    }

	    // handel integer buffer case

	    else if (imageBuffer instanceof DataBufferInt)
	    {
	      imageInts = ((DataBufferInt)imageBuffer).getData();
	    }

	    // if it's some other type, throw

	    else
	    {
	      throw new IllegalArgumentException(
	        "Unsupported BufferedImage data buffer type: " +
	        imageBuffer.getDataType());
	    }

	    // create the video picture and get it's underling buffer

	    final AtomicReference<JNIReference> ref = new AtomicReference<JNIReference>(null);
	    IVideoPicture resamplePicture = null;
	    try
	    {
	      IVideoPicture picture = IVideoPicture.make(IPixelFormat.Type.ARGB, image.getWidth(),
	          image.getHeight());

	      ByteBuffer pictureByteBuffer = picture.getByteBuffer(ref);

	      if (imageInts != null)
	      {
	        pictureByteBuffer.order(ByteOrder.BIG_ENDIAN);
	        IntBuffer pictureIntBuffer = pictureByteBuffer.asIntBuffer();
	        pictureIntBuffer.put(imageInts);
	      }
	      else
	      {
	        pictureByteBuffer.put(imageBytes);
	      }
	      pictureByteBuffer = null;
	      picture.setComplete(true, IPixelFormat.Type.ARGB, image.getWidth(),
	          image.getHeight(), timestamp);

	      // resample as needed
	      /*if (willResample()) {
	        resamplePicture = picture;
	        picture = resample(resamplePicture, mToPictureResampler);
	      }*/
	      return picture;
	    }
	    finally
	    {
	      if (resamplePicture != null)
	        resamplePicture.delete();
	      if (ref.get() != null)
	        ref.get().delete();
	    }
	  }
	 
}