package save;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.ShortBuffer;
import java.util.concurrent.TimeUnit;

import player.Player;
import view.PlayerMainPanel;
import view.StatusDialog;

import com.xuggle.mediatool.IMediaReader;
import com.xuggle.mediatool.IMediaTool;
import com.xuggle.mediatool.IMediaWriter;
import com.xuggle.mediatool.MediaToolAdapter;
import com.xuggle.mediatool.ToolFactory;
import com.xuggle.mediatool.event.IAudioSamplesEvent;
import com.xuggle.mediatool.event.IVideoPictureEvent;
import com.xuggle.xuggler.ICodec;
import com.xuggle.xuggler.IContainer;
import com.xuggle.xuggler.IStream;
import com.xuggle.xuggler.IStreamCoder;

import controller.OSValidator;
import controller.PlayerUIListener;

/**
 * This class handles the saving process.
 * This contains converting and subtitle adding.
 * @author Johann Houszka 0625523
 *
 */
public class SaveVideo {

	private String inputFilename = "";
	private String outputFilename = "";
	private String subOptions;
	private String subtitleFile;
	private String correctSubtitleFile;
	private int subtitleLang;
	private PlayerUIListener listener;
	private PlayerMainPanel mainPanel;
	private String videoOptions;
	private Player player;
	private String subLang;
	private StatusDialog saveProgress;
	private String outString, outStringXuggler;
	private OSValidator osValidator;
	private boolean check;
	IMediaReader mediaReader;
	IMediaWriter mediaWriter;
	IMediaTool imageMediaTool, audioVolumeMediaTool;
	boolean cancelProcess = false;

	/**
	 * Constructor for this class. Sets parameters.
	 * @param player
	 * @param in
	 * @param out
	 * @param suboptions
	 * @param videooption
	 * @param subtitleFile
	 * @param subtitleLang
	 * @param listener
	 * @param mainPanel
	 */
	public SaveVideo(Player player, String in, String out, String suboptions, String videooption, String subtitleFile, int subtitleLang, boolean check, PlayerUIListener listener, PlayerMainPanel mainPanel) {
		this.player = player;
		this.inputFilename = in;
		this.outputFilename = out;
		this.subOptions = suboptions;
		this.videoOptions = videooption;
		this.subtitleFile = subtitleFile;
		this.subtitleLang = subtitleLang;
		//this.correctSubtitleFile = listener.getSubtitles().getFile();
		this.check = check;
		this.listener = listener;
		this.mainPanel = mainPanel;
		
		System.out.println(this.subtitleFile);
		
		osValidator = new OSValidator();
		
		subLang = getSubtitleLanguage(this.subtitleLang);
		
		Thread t = new Thread(new Runnable() {
			public void run() {
				SaveAndConvert();
			}
			
		});
		t.start();
	}

	/**
	 * Build Filename to given parameters.
	 * @param outputFilename
	 * @param videooptions
	 * @return
	 */
	public String buildFileName(String outputFilename, String videooptions) {
		String outString = "";
		if(videooptions.toLowerCase().equals("mp4")) {
			if(!outputFilename.toLowerCase().endsWith("mp4")) {
				String[] check = outputFilename.split("\\.");
				outString = check[0] + ".mp4";
			}
			else {
				outString = outputFilename;
			}
		}
		else if (videooptions.toLowerCase().equals("mkv")) {
			if(!outputFilename.toLowerCase().endsWith("mkv")) {
				String[] check = outputFilename.split("\\.");
				System.out.println("Split: " + check[0] + "::" + check[1]);
				outString = check[0] + ".mkv";
			}
			else {
				outString = outputFilename;
			}
		}
		return outString;
	}
	
	/**
	 * Xuggler cant convert video to .mkv, so we have to set the file-ending
	 * of the outputVideo to .mp4
	 * @param outputFilename
	 * @param videooptions
	 * @return outString
	 */
	public String buildXugglerFileName(String outputFilename, String videooptions) {
		String outString = "";
		String[] check = outputFilename.split("\\.");
		outString = check[0] + ".mp4";

		return outString;
	}

	/**
	 * Delete temporary files
	 * @param path
	 */
	public void deleteFile(String path) {
		File file = new File(path);
        if(file.exists()){
        	System.out.println("File " + path + " existiert. Lschen starten...");
            file.delete();
        }
        if(file.getAbsoluteFile().exists()) {
        	System.out.println("File " + file.getAbsoluteFile().toString() + " existiert. Lschen starten...");
        	file.getAbsoluteFile().delete();
        }
	}
	
	/**
	 * Helper method 
	 * get subtitle options for language
	 */
	private String getSubtitleLanguage(int subLang) {
		String sub;
		switch (subLang) {
        case 0:  sub = "ger";
                 break;
        case 1:  sub = "eng";
                 break;
        case 2:  sub = "fre";
                 break;
        case 3:  sub = "ita";
                 break;
        case 4:  sub = "spa";
        		 break;
        default: sub = "ger";
                 break;
		}
		return sub;
	}
	
	/**
	 * Masks input string for command line
	 * @param path
	 * @return
	 */
	public String quoteWhitespaces(String path) {
		String _temp[] = path.split("///");
		System.out.println(_temp);
		for (int i = 0; i < _temp.length; i++) {
			if(_temp[i].contains(" ")) {
				_temp[i] ="\" \""+ _temp[i] + "\" \""; //_temp[i].replace(" ", "\" \"");
			}
			System.out.println(_temp[i]);
		}
		//return path.replaceAll(" ", "\" \"");
		String corpath = "";
		for (int i =0; i < _temp.length; i++) {
			corpath += _temp[i];
		}
		System.out.println("CorPATH: " + corpath);
		return corpath;
	}

	/**
	 * Methods saves and converts to video for given
	 * input parameters.
	 * @author Johann Houszka
	 */
	public synchronized void SaveAndConvert() {
		saveProgress = new StatusDialog(this.mainPanel, mainPanel.getMovieDuration(), this);

		outString = buildFileName(this.outputFilename, this.videoOptions);
		outStringXuggler = buildXugglerFileName(this.outputFilename, this.videoOptions);
		//measure time it takes to save file
		long start = System.currentTimeMillis();
		//save file with titles and graphics
		//xuggler can only save .mp4 files no mkv)
		System.out.println("InputFile: " + inputFilename);
		System.out.println("OutputFile: " + outputFilename);
		System.out.println("OutputFile-Xuggler: " + outStringXuggler);
		System.out.println("OutputFile-Real: " + outString);
		//getInfoAboutInputFile(inputFilename);
		if(!check) {
			this.save(inputFilename, outStringXuggler);
			if(this.cancelProcess == false) {
				System.out.println("Movie export finished (Title,Graphics) :: Duration: " + (System.currentTimeMillis() - start) + "ms");
				saveProgress.updateInfo("Finishing...", mainPanel.getMovieDuration()-1);
			}
			if(this.cancelProcess) {
				deleteFile(outStringXuggler);
				deleteFile(outString);
			}
		}
		//getInfoAboutInputFile(outStringXuggler);

		/*
		 ************************************************************
		 ************************************************************
		 * CONVERT TO MKV
		 ************************************************************ 
		 ************************************************************
		 */
		if(this.videoOptions.toLowerCase().equals("mkv") && this.subOptions.equals("No Subtitles") && this.cancelProcess == false) {
			if(this.check) {this.outStringXuggler = this.inputFilename;}
			Process p;
			try {
				System.out.println("Convert MP4 to MKV.");
				String ffmpegPath = this.player.getSettings().getLibPath("ffmpeg");
				//-b 3000k ...bitrate 
				p = Runtime.getRuntime().exec(ffmpegPath + " -i  " + outStringXuggler + " " + outString);
				
				//streams need to be closed or p.waitFor() can get into a deadlock
				p.getErrorStream().close();
				p.getInputStream().close();
				p.getOutputStream().close();
				p.waitFor();
				//outStringXuggler = outString;
				
				System.out.println("Convert MP4 to MKV finished. ("+outStringXuggler+")");
				deleteFile(outStringXuggler);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} 
		}
		
		
		/*
		 ************************************************************
		 ************************************************************
		 * SUBTITLE OPTIONS
		 ************************************************************ 
		 ************************************************************
		 */
		if(this.subOptions.equals("No Subtitles") && this.cancelProcess == false) {
			System.out.println("No Subtitles.");
		}
		else if(this.subOptions.equals("Render") && this.cancelProcess == false) {
			if(this.check) {this.outStringXuggler = this.inputFilename;}
			System.out.println("Subtitle Option: Render Subtitles into video.");
			String commandConvert = "";
			String commandRender = "";
			start = System.currentTimeMillis();
			this.correctSubtitleFile = listener.getSubtitles().getFile();
			
			//add subtitles via ffmpeg  
			try {
				Process p;
				//getOS and change commandString for differentOS
				if(osValidator.isMacOS()) {
					//subtitle file must be *.ass-file
					commandConvert = this.player.getSettings().getLibPath("ffmpeg") + " -i " + this.correctSubtitleFile + " -y subtitles/test.ass";
					commandRender = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -strict -2 -vf ass=subtitles/test.ass -y " + outString;

					p = Runtime.getRuntime().exec(commandConvert);
					p.waitFor();
				}
				else if(osValidator.isWindowsOS()) {
					//subtitle file must be *.srt-file
					//commandConvert = this.player.getSettings().getLibPath("ffmpeg") + " -i " + this.subtitleFile + " -y subtitles/test.ass";
					commandRender = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -i " + this.subtitleFile + " -y " + outString;

				}
				p = Runtime.getRuntime().exec(commandRender);
				
				//streams need to be closed or p.waitFor() can get into a deadlock
				p.getErrorStream().close();
			    p.getInputStream().close();
			    p.getOutputStream().close();
			    p.waitFor();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			System.out.println("Subtitle File added to container :: Duration: " + (System.currentTimeMillis() - start) + "ms");	
			if(this.videoOptions.toLowerCase().equals("mkv") && !this.check) deleteFile(outStringXuggler);
		}
		else if(this.subOptions.equals("File") && this.cancelProcess == false) {
			if(this.check) {this.outStringXuggler = this.inputFilename;}
			String command = "";
			this.correctSubtitleFile = listener.getSubtitles().getFile();
			
			//getOS and change commandString for differentOS
			if(osValidator.isMacOS()) {
				//add subtitles with transcoding (mp4 to mkv)
				if(this.videoOptions.equals("mkv")) 
				{
					command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -i " + this.correctSubtitleFile + " -vcodec copy -acodec copy -scodec srt -map 0:0 -map 0:1 -map 1:0 -y " + outString;
					//command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -newsubtitle " + correctSubtitleFile + " " + outString;
					//System.out.println(command);
					//command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -sub_charenc ISO-8859-1 -i " + subtitleFile + " -vcodec h264 -acodec ac3 -scodec srt -metadata:s:s:0 language=" + subLang + " -y " + outString;
				}
				//add subtitles without transcoding (mp4 to mp4)
				else if(this.videoOptions.equals("mp4")) 
				{
					command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -sub_charenc CP1252 -i " + correctSubtitleFile + " -vcodec copy -acodec copy -scodec mov_text -metadata:s:s:0 language=" + subLang + " -y " + outString;
					System.out.println(command);
				}
			}
			else if(osValidator.isWindowsOS()) {
				//add subtitles with transcoding (mp4 to mkv)
				if(this.videoOptions.equals("mkv")) 
				{
					command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -sub_charenc ISO-8859-1 -i " + subtitleFile + " -vcodec h264 -acodec ac3 -scodec srt -metadata:s:s:0 language=" + subLang + " " + outString;
				}
				//add subtitles without transcoding (mp4 to mp4)
				else if(this.videoOptions.equals("mp4")) 
				{
					command = this.player.getSettings().getLibPath("ffmpeg") + " -i " + outStringXuggler + " -sub_charenc WINDOWS-1252 -i " + subtitleFile + " -vcodec mpeg4 -acodec mp3 -scodec mov_text -metadata:s:s:0 language=" + subLang + " " + outString;
				}
			}
			
			System.out.println("Subtitle Options: Add extra Subtitle-File to video.");
			long start1 = System.currentTimeMillis();
			try {
				Process p;
				p = Runtime.getRuntime().exec(command);

				//streams need to be closed or p.waitFor() can get into a deadlock
		        p.getErrorStream().close();
		        p.getInputStream().close();
		        p.getOutputStream().close();
				p.waitFor();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			System.out.println("Subtitle File added to container :: Duration: " + (System.currentTimeMillis() - start1) + "ms");
			if(this.videoOptions.toLowerCase().equals("mkv") && !this.check) deleteFile(outStringXuggler);
		}
		saveProgress.updateInfo("Saved in "+((System.currentTimeMillis() - start)/1000)+"s", mainPanel.getMovieDuration());
		
	}


	/**
	 * Main method for saving video.
	 * @param in
	 * @param out
	 */
	public void save(String in, String out) {
		
		// create a media reader
		mediaReader = ToolFactory.makeReader(in);
		mediaReader.getContainer().getDuration();
		// configure it to generate BufferImages
		mediaReader.setBufferedImageTypeToGenerate(BufferedImage.TYPE_3BYTE_BGR);
		
		mediaWriter = ToolFactory.makeWriter(out, mediaReader);
		
		imageMediaTool = new StaticImageMediaTool(listener, saveProgress, mainPanel);
		audioVolumeMediaTool = new VolumeAdjustMediaTool(0.8);

		// create a tool chain:
		// reader -> addStaticImage -> reduceVolume -> writer
		mediaReader.addListener(imageMediaTool);
		imageMediaTool.addListener(audioVolumeMediaTool);
		audioVolumeMediaTool.addListener(mediaWriter);

		
		while (mediaReader.readPacket() == null && this.cancelProcess == false) {
			
		}

	}
	
	/**
	 * Closes reader and writer stream
	 * if user hits cancel button.
	 * @author Johann Houszka
	 */
	public void stopSavingProcess() {
		this.cancelProcess = true;
		saveProgress.updateInfo("Cancel saving process...", 10);
		try 
		{
			this.audioVolumeMediaTool.removeListener(mediaWriter);
			this.imageMediaTool.removeListener(audioVolumeMediaTool);
			this.mediaReader.removeListener(imageMediaTool);
			this.mediaReader.close();
			this.mediaWriter.flush();
			this.mediaWriter.close();		
			
		} catch(Exception ex) {
			System.out.println("Exception beim Canceln des Prozesses wurde geworfen!");
		}
	}
	
	/**
	 * Prints information about the input file to the console.
	 * @author Johann Houszka
	 * @param in
	 */
	public void getInfoAboutInputFile(String in) {
		
		//get Infos about the Input File
		IContainer container = IContainer.make();
			 
		// we attempt to open up the container
			 
		container.open(in, IContainer.Type.READ, null);
		int numStreams = container.getNumStreams();
				 
		// query for the total duration
		long duration = container.getDuration();
		 
		// query for the file size
		long fileSize = container.getFileSize();
		 
		// query for the bit rate
		long bitRate = container.getBitRate();
		
		System.out.println("Number of streams: " + numStreams);
		System.out.println("Duration (ms): " + duration);
		System.out.println("File Size (bytes): " + fileSize); 
		System.out.println("Bit Rate: " + bitRate);
		
		// iterate through the streams to print their meta data
        for (int i=0; i < numStreams; i++) {
            
            // find the stream object
            IStream stream = container.getStream(i);
            
            // get the pre-configured decoder that can decode this stream;
            IStreamCoder coder = stream.getStreamCoder();
            
            System.out.println("*** Start of Stream Info ***");
            
            System.out.printf("stream %d: ", i);
            System.out.printf("type: %s; ", coder.getCodecType());
            System.out.printf("codec: %s; ", coder.getCodecID());
            System.out.printf("duration: %s; ", stream.getDuration());
            System.out.printf("start time: %s; ", container.getStartTime());
            System.out.printf("timebase: %d/%d; ",
                 stream.getTimeBase().getNumerator(),
                 stream.getTimeBase().getDenominator());
            System.out.printf("coder tb: %d/%d; ",
                 coder.getTimeBase().getNumerator(),
                 coder.getTimeBase().getDenominator());
            System.out.println();
            
            if (coder.getCodecType() == ICodec.Type.CODEC_TYPE_AUDIO) {
                System.out.printf("sample rate: %d; ", coder.getSampleRate());
                System.out.printf("channels: %d; ", coder.getChannels());
                System.out.printf("format: %s", coder.getSampleFormat());
            } 
            else if (coder.getCodecType() == ICodec.Type.CODEC_TYPE_VIDEO) {
                System.out.printf("width: %d; ", coder.getWidth());
                System.out.printf("height: %d; ", coder.getHeight());
                System.out.printf("format: %s; ", coder.getPixelType());
                System.out.printf("frame-rate: %5.2f; ", coder.getFrameRate().getDouble());
            }
            
            System.out.println();
            System.out.println("*** End of Stream Info ***");
            
        }
				
	}
}

/**
 * Class for saving frames to video.
 * @author Johann Houszka, Matthias Eigner
 *
 */
class StaticImageMediaTool extends MediaToolAdapter {

	private PlayerUIListener listener;
	private StatusDialog saveProgress;
	//private StatusMonitor smon;
	private PlayerMainPanel mainPanel;

	public StaticImageMediaTool(PlayerUIListener listener, StatusDialog saveProgress, PlayerMainPanel mainPanel) {
			this.listener = listener;
			this.saveProgress = saveProgress;
			//this.smon = smon;
			this.mainPanel = mainPanel;

	}

	private int[] data;
	/* (non-Javadoc)
	 * @see com.xuggle.mediatool.MediaToolAdapter#onVideoPicture(com.xuggle.mediatool.event.IVideoPictureEvent)
	 */
	@Override
	public void onVideoPicture(IVideoPictureEvent event) {
		//System.out.println((event.getTimeStamp()/1000)+"-"+mainPanel.getMovieDuration());
		this.saveProgress.updateInfo((event.getTimeStamp(TimeUnit.MILLISECONDS))/(mainPanel.getMovieDuration()/100)+"% done ", (event.getTimeStamp()/1000));
		//this.smon.updateInfo((event.getTimeStamp(TimeUnit.MILLISECONDS))/(mainPanel.getMovieDuration()/100)+"% done ", (event.getTimeStamp()/1000));
		
		BufferedImage image = event.getImage();
		
		data = image.getRGB(0,0,image.getWidth(), image.getHeight(), null, 0, image.getWidth());
		
		data = this.listener.getActiveEffect().applyEffect(data);
		
		image.setRGB(0, 0, image.getWidth(), image.getHeight(), data, 0, image.getWidth());
		
		// get the graphics for the image
		Graphics2D g = image.createGraphics();
		
		// add Graphics
		// devided by 1000 because getImageAtPosition needs milli seconds and getTimeStamp returnes micro seconds
		g.drawImage(this.listener.getImages().getImageAtPosition(event.getTimeStamp()/1000, image.getWidth(), image.getHeight()), 0, 0, image.getWidth(), image.getHeight(), null);
		
		// add Titles
		// devided by 1000 because getImageAtPosition needs milli seconds and getTimeStamp returnes micro seconds
		g.drawImage(this.listener.getTitles().getImageAtPosition(event.getTimeStamp()/1000, image.getWidth(), image.getHeight()), 0, 0, image.getWidth(), image.getHeight(), null);
		
		// call parent which will pass the video onto next tool in chain
		super.onVideoPicture(event);

	}
}

/**
 * Class to adjust output volume of video.
 * @author Johann Houszka, Matthias Eigner
 *
 */
class VolumeAdjustMediaTool extends MediaToolAdapter {

	// the amount to adjust the volume by
	private double mVolume;

	public VolumeAdjustMediaTool(double volume) {
		mVolume = volume;
	}

	@Override
	public void onAudioSamples(IAudioSamplesEvent event) {

		// get the raw audio bytes and adjust it's value
		ShortBuffer buffer = event.getAudioSamples().getByteBuffer()
				.asShortBuffer();

		for (int i = 0; i < buffer.limit(); ++i) {
			buffer.put(i, (short) (buffer.get(i) * mVolume));
		}

		// call parent which will pass the audio onto next tool in chain
		super.onAudioSamples(event);

	}

}
