package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import player.Player;
import uk.co.caprica.vlcj.binding.LibVlcConst;
import controller.PlayerUIListener;

/**
 * Class for constructing the ControlsPanel.
 * @author Johann Houszka 0625523
 * 
 */
public class PlayerControlsPanel extends ImagePanel {

	/**
	 * Default ID
	 */
	private static final long serialVersionUID = 1L;

	private Player player;
	private JButton nav_start, nav_stop, nav_back, nav_forw, nav_volumemute, nav_volumefull;
	private JSlider volumeSlider, source;
	public JPanel controlsPanel;
	private PlayerUIListener uiListener;
	private ImageIcon iconplay, iconpause, iconmute, iconvol, iconplay_active, iconbegin_active,iconpause_active, iconforw_active,iconbackw_active;
	private JButton separator1, separator2, separator3, separator4;

	private JPanel panelleft, panelright;
	private JLabel timelabel;

	// slider status
	private int tempvol = -1;

	// playing status
	private int status;
	public static final int STATUS_PLAY = 0;
	public static final int STATUS_PAUSE = 1;

	// volume status
	private int volstatus;
	public static final int STATUS_MUTE = 2;
	public static final int STATUS_VOL = 3;

	/**
	 * Constructor of this class.
	 * @param player
	 * @param mainPanel
	 */
	public PlayerControlsPanel(Player player, PlayerMainPanel mainPanel) {
		super("img/layout/controls_background.png");
		this.player = player;
		uiListener = mainPanel.getUIListener();

		iconplay = new ImageIcon("img/icons/controls_play.png");
		iconplay_active = new ImageIcon("img/icons/controls_play_active.png");
		iconpause = new ImageIcon("img/icons/controls_pause.png");
		iconmute = new ImageIcon("img/icons/controls_volume_0.png");
		iconvol = new ImageIcon("img/icons/controls_volume_100.png");
		iconbegin_active = new ImageIcon("img/icons/controls_begin_active.png");
		iconpause_active = new ImageIcon("img/icons/controls_pause_active.png");
		iconforw_active = new ImageIcon("img/icons/controls_forw_active.png");
		iconbackw_active = new ImageIcon("img/icons/controls_backw_active.png");

		status = PlayerControlsPanel.STATUS_PAUSE;
		volstatus = PlayerControlsPanel.STATUS_VOL;

		createUI();
	}

	/**
	 * Creates UI for the ControlsPanel.
	 */
	private void createUI() {

		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.anchor = GridBagConstraints.WEST;
		setMinimumSize(new Dimension(700, 45));
		setMaximumSize(new Dimension(1500, 45));
		setLayout(new BorderLayout());

		panelleft = new JPanel();
		panelleft.setOpaque(false);
		panelleft.setLayout(new GridBagLayout());
		panelleft.setMinimumSize(new Dimension(200, 45));
		panelleft.setMaximumSize(new Dimension(400, 45));
		panelleft.setBackground(Color.red);

		panelright = new JPanel();
		panelright.setOpaque(false);
		panelright.setLayout(new GridBagLayout());
		panelright.setMinimumSize(new Dimension(200, 45));
		panelright.setMaximumSize(new Dimension(200, 45));

		// Separator
		separator1 = new JButton();
		separator1.setIcon(new ImageIcon("img/layout/controls_seperator.png"));
		separator1.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		separator1.setEnabled(true);
		separator1.setBorderPainted(false); 
		separator1.setContentAreaFilled(false); 
		separator1.setFocusPainted(false); 
		separator1.setOpaque(false);
		// Separator
		separator2 = new JButton();
		separator2.setIcon(new ImageIcon("img/layout/controls_seperator.png"));
		separator2.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		separator2.setEnabled(true);
		separator2.setBorderPainted(false); 
		separator2.setContentAreaFilled(false); 
		separator2.setFocusPainted(false); 
		separator2.setOpaque(false);
		// Separator
		separator3 = new JButton();
		separator3.setIcon(new ImageIcon("img/layout/controls_seperator.png"));
		separator3.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		separator3.setEnabled(true);
		separator3.setBorderPainted(false); 
		separator3.setContentAreaFilled(false); 
		separator3.setFocusPainted(false); 
		separator3.setOpaque(false);
		// Separator
		separator4 = new JButton();
		separator4.setIcon(new ImageIcon("img/layout/controls_seperator.png"));
		separator4.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		separator4.setEnabled(true);
		separator4.setBorderPainted(false); 
		separator4.setContentAreaFilled(false); 
		separator4.setFocusPainted(false); 
		separator4.setOpaque(false);

		/*
		 * buttons for controlpanel
		 */
		int button_margin_top = 4;
		int button_margin_left = 0;
		int button_margin_right = 0;
		int button_margin_bottom = 0;
		int button_inset = -6;
		
		nav_stop = new JButton();
		nav_stop.setBorderPainted(true);
		nav_stop.setIcon(new ImageIcon("img/icons/controls_begin.png"));
		nav_stop.setBorder(BorderFactory.createEmptyBorder(button_margin_top, button_margin_left, button_margin_bottom, button_margin_right)); // important
		nav_stop.setFocusable(false);
		nav_stop.setBorderPainted(false); 
		nav_stop.setContentAreaFilled(false); 
		nav_stop.setFocusPainted(false); 
		nav_stop.setOpaque(false);
		// What button looks like normally after setSelected( true ).
		nav_stop.setSelectedIcon (iconbegin_active);
		// what button looks like pressed.
		nav_stop.setPressedIcon(iconbegin_active);
		// what button looks like when mouse hovers over it.
		nav_stop.setRolloverIcon(iconbegin_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		nav_stop.setRolloverSelectedIcon(iconbegin_active);

		
		
		nav_stop.setActionCommand("control_stop");
		nav_stop.addActionListener(uiListener);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(button_inset, 0, 0, 0);
		panelleft.add(nav_stop, gbc);
		
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		panelleft.add(separator1, gbc);
		
		nav_back = new JButton();
		nav_back.setIcon(new ImageIcon("img/icons/controls_back.png"));
		nav_back.setToolTipText("Move back");
		nav_back.setBorder(BorderFactory.createEmptyBorder(button_margin_top, button_margin_left, button_margin_bottom, button_margin_right)); // important
		nav_back.setFocusable(false);
		nav_back.setBorderPainted(false); 
		nav_back.setContentAreaFilled(false); 
		nav_back.setFocusPainted(false); 
		nav_back.setOpaque(false);
		// What button looks like normally after setSelected( true ).
		nav_back.setSelectedIcon (iconbackw_active);
		// what button looks like pressed.
		nav_back.setPressedIcon(iconbackw_active);
		// what button looks like when mouse hovers over it.
		nav_back.setRolloverIcon(iconbackw_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		nav_back.setRolloverSelectedIcon(iconbackw_active);
		
		nav_back.setActionCommand("control_backward");
		nav_back.addActionListener(uiListener);
		
		gbc.gridx = 2;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(button_inset, 0,0,0);
		panelleft.add(nav_back, gbc);

		gbc.gridx = 4;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		panelleft.add(separator2, gbc);

		nav_start = new JButton();
		nav_start.setIcon(iconplay);
		nav_start.setBorder(BorderFactory.createEmptyBorder(button_margin_top, button_margin_left, button_margin_bottom, button_margin_right)); // important
		nav_start.setFocusable(false);
		nav_start.setBorderPainted(false); 
		nav_start.setContentAreaFilled(false); 
		nav_start.setFocusPainted(false); 
		nav_start.setOpaque(false);
		// What button looks like normally after setSelected( true ).
		nav_start.setSelectedIcon (iconplay_active);
		// what button looks like pressed.
		nav_start.setPressedIcon(iconplay_active);
		// what button looks like when mouse hovers over it.
		nav_start.setRolloverIcon(iconplay_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		nav_start.setRolloverSelectedIcon(iconplay_active);

	
		nav_start.setActionCommand("control_play");
		nav_start.addActionListener(uiListener);

		
		gbc.gridx = 5;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(button_inset, 0,0,0);
		panelleft.add(nav_start, gbc);
		

		gbc.gridx = 6;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		panelleft.add(separator3, gbc);

		nav_forw = new JButton();
		nav_forw.setIcon(new ImageIcon("img/icons/controls_next.png"));
		nav_forw.setBorder(BorderFactory.createEmptyBorder(button_margin_top, button_margin_left, button_margin_bottom, button_margin_right)); // important
		nav_forw.setFocusable(false);
		nav_forw.setBorderPainted(false); 
		nav_forw.setContentAreaFilled(false); 
		nav_forw.setFocusPainted(false); 
		nav_forw.setOpaque(false);
		// What button looks like normally after setSelected( true ).
		nav_forw.setSelectedIcon (iconforw_active);
		// what button looks like pressed.
		nav_forw.setPressedIcon(iconforw_active);
		// what button looks like when mouse hovers over it.
		nav_forw.setRolloverIcon(iconforw_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		nav_forw.setRolloverSelectedIcon(iconforw_active);
		
		
		nav_forw.setActionCommand("control_forward");
		nav_forw.addActionListener(uiListener);
		
		gbc.gridx = 7;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(button_inset, 0,0,0);
		panelleft.add(nav_forw, gbc);

		gbc.gridx = 8;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		panelleft.add(separator4, gbc);
		
		timelabel = new JLabel();
		timelabel.setBorder(BorderFactory.createEmptyBorder(6, 6, 0, 0));
		timelabel.setText("00:00:00");
		timelabel.setForeground(Color.white);
		timelabel.setFont(new Font(this.getFont().getFontName(), 4, 16));
	
		gbc.gridx = 9;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		panelleft.add(timelabel, gbc);

		nav_volumemute = new JButton();
		nav_volumemute.setIcon(iconmute);
		nav_volumemute.setFocusable(false);
		//nav_volumemute.setBorder(BorderFactory.createEmptyBorder(5, 30, 5, 2));
		nav_volumemute.setBorder(BorderFactory.createEmptyBorder(5, 2, 5, 5));
		nav_volumemute.setBorderPainted(false); 
		nav_volumemute.setContentAreaFilled(false); 
		nav_volumemute.setFocusPainted(false); 
		nav_volumemute.setOpaque(false);
		
		nav_volumemute.setActionCommand("control_mute");
		nav_volumemute.addActionListener(uiListener);

		gbc.gridx = 9;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(-5, 50, 1, 1);
		panelright.add(nav_volumemute, gbc);

		volumeSlider = new JSlider();
		volumeSlider.setOrientation(JSlider.HORIZONTAL);
		volumeSlider.setMinimum(LibVlcConst.MIN_VOLUME);
		volumeSlider.setMaximum(LibVlcConst.MAX_VOLUME);
		volumeSlider.setMinimumSize(new Dimension(100, 30));
		volumeSlider.setToolTipText("Change volume");
		volumeSlider.setFocusable(false);
		volumeSlider.setOpaque(false);
		
		tempvol = 50;

		volumeSlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				source = (JSlider) e.getSource();
				player.getMediaPlayer().setVolume(source.getValue());
				if (source.getValue() == 0) {
					setVolStatus(STATUS_MUTE);
				} else {
					if (player.getMediaPlayer().isMute())
						player.getMediaPlayer().mute();
					setVolStatus(STATUS_VOL);
				}
			}
		});

		gbc.gridx = 10;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(0, 1, 1, 1);
		panelright.add(volumeSlider, gbc);
		
		nav_volumefull = new JButton();
		nav_volumefull.setIcon(iconvol);
		nav_volumefull.setBorder(BorderFactory.createEmptyBorder(5, 2, 5, 5));
		nav_volumefull.setFocusable(false);
		nav_volumefull.setBorderPainted(false); 
		nav_volumefull.setContentAreaFilled(false); 
		nav_volumefull.setFocusPainted(false); 
		nav_volumefull.setOpaque(false);

		
		nav_volumefull.setActionCommand("control_vol");
		nav_volumefull.addActionListener(uiListener);

		gbc.gridx = 11;
		gbc.gridy = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(-5, 1, 1, 1);
		panelright.add(nav_volumefull, gbc);

		add(BorderLayout.WEST, panelleft);
		add(BorderLayout.EAST, panelright);
	}

	
	/*
	 ************************************************************
	 ************************************************************
	 * Methods used in this class.
	 ************************************************************ 
	 ************************************************************
	 */
	
	public void setControlsDisabled(boolean status) {
		Component[] controlComp = this.panelleft.getComponents();
		for (int i = 0; i < controlComp.length; i++) {
			controlComp[i].setEnabled(status);
		}
		Component[] controlCompR = this.panelright.getComponents();
		for (int i = 0; i < controlCompR.length; i++) {
			controlCompR[i].setEnabled(status);
		}
	}
	
	/**
	 * Set Time in Label.
	 * @param timelabel
	 */
	public void setTime(JLabel timelabel) {
		this.timelabel = timelabel;
	}

	/**
	 * Returns time.
	 * @return timelabel
	 */
	public JLabel getTime() {
		return timelabel;
	}

	/**
	 * Set menu icons to play. 
	 * Video is playing and gets set to paused.
	 */
	public void setPaused() {
		this.nav_start.setIcon(iconplay);
		this.nav_start.setSelectedIcon (iconplay_active);
		// what button looks like pressed.
		this.nav_start.setPressedIcon(iconplay_active);
		// what button looks like when mouse hovers over it.
		this.nav_start.setRolloverIcon(iconplay_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		this.nav_start.setRolloverSelectedIcon(iconplay_active);
		this.setStatus(STATUS_PAUSE);
	}

	/**
	 * Set menu icons to pause.
	 * Video is pausing and gets set to play.
	 */
	public void setPlaying() {
		this.nav_start.setIcon(iconpause);
		this.nav_start.setSelectedIcon (iconpause_active);
		// what button looks like pressed.
		this.nav_start.setPressedIcon(iconpause_active);
		// what button looks like when mouse hovers over it.
		this.nav_start.setRolloverIcon(iconpause_active);
		// what button looks like when mouse hovers over it after setSelected( true ).
		this.nav_start.setRolloverSelectedIcon(iconpause_active);
		this.setStatus(STATUS_PLAY);
	}

	/**
	 * Set audio to mute.
	 */
	public void setMute() {
		this.setVolStatus(STATUS_MUTE);
		if (this.source != null) {
			tempvol = this.source.getValue();
			this.source.setValue(0);
		} else
			volumeSlider.setValue(0);

	}
	
	/**
	 * Set temporary volume. Used for audio controls.
	 */
	public void setTempVolume() {
		this.source.setValue(tempvol);
	}

	/**
	 * Set volume to given value.
	 */
	public void setVolume() {
		this.setVolStatus(STATUS_VOL);
		if (this.source != null)
			volumeSlider.setValue(LibVlcConst.MAX_VOLUME);	
		else
			this.source.setValue(tempvol);		
	}

	/**
	 * Return status.
	 * @return status
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * Set status of video (PAUSED/PLAYING).
	 * @param status
	 */
	public void setStatus(int status) {
		this.status = status;
	}

	/**
	 * Returns the audio status.
	 * @return
	 */
	public int getVolStatus() {
		return volstatus;
	}

	/**
	 * Set the audio status (MUTE/ON).
	 * @param volstatus
	 */
	public void setVolStatus(int volstatus) {
		this.volstatus = volstatus;
	}

	/**
	 * Calculate time to display in menu.
	 * @return String timetext
	 */
	public String calcTime() {
		long time = player.getMediaPlayer().getTime();
		long seconds = time / 1000;
		long minutes = seconds / 60;
		long hours = minutes / 60;
		String timeText = "";
		if((seconds%60) <10 && (minutes%60 >= 10)) timeText = "0" + String.valueOf(hours) + ":" + String.valueOf(minutes%60)
				+ ": 0" + String.valueOf(seconds%60);
		else if ((minutes %60) <10 && (seconds %60) >= 10) timeText =  "0" + String.valueOf(hours) + ":0" + String.valueOf(minutes%60)
		+ ":" + String.valueOf(seconds%60);
		else if((seconds%60) < 10 && (minutes%60) <10) timeText =  "0" + String.valueOf(hours) + ":0" + String.valueOf(minutes%60)
				+ ":0" + String.valueOf(seconds%60);
		else timeText =  "0" + String.valueOf(hours) + ":" + String.valueOf(minutes%60)
				+ ":" + String.valueOf(seconds%60);
		
		return timeText;
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
//	@Override
//	public void actionPerformed(ActionEvent e) {
//		if (e.getActionCommand() == "control_play") {
//			nav_start.setIcon(iconplay_active);
//		}
//		
//	}
}
