package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import player.Player;
import controller.PlayerUIListener;
import effects.EffectFactory;

/**
 * Class for drawing EffectPanel and adding Effect thumbs.
 * @author Johann Houszka 0625523
 *
 */
public class PlayerEffectsPanel extends ImagePanel {

	/**
	 * Default ID
	 */
	private static final long serialVersionUID = 1L;
	
	private PlayerUIListener uiListener;
	public JPanel effectsPanel;
	private EffectFactory effectFactory;
	private JScrollPane scrollPane;
	private BufferedImage effects_selected, imagePanelBG, selectedPanelBG;
	private List<ImagePanel> imagelist, selectedborderlist;
	private Player player;

	/**
	 * Constructor of this class. Instantiates the EffectsPanel.
	 * @param Player player
	 * @param PlayerMainPanel mainPanel
	 */
	public PlayerEffectsPanel(Player player, PlayerMainPanel mainPanel) {
		super("img/layout/effects_background.png");
		this.uiListener = mainPanel.getUIListener();
		this.effectFactory = new EffectFactory(player);
		this.imagelist = new ArrayList<ImagePanel>();
		this.selectedborderlist = new ArrayList<ImagePanel>();
		this.player = player;
		
		try {
			URL url = getClass().getResource("/img/layout/effects_effect_border_selectedbig.png");
			if (url == null)
			    System.out.println( "Could not find image!" );
			this.effects_selected = ImageIO.read(getClass().getResource("/img/layout/effects_effect_border_selectedbig.png"));
			this.imagePanelBG = ImageIO.read(getClass().getResource("/img/layout/effects_effect_borderbig.png"));
			this.selectedPanelBG = ImageIO.read(getClass().getResource("/img/layout/effects_effect_border_selectedbig.png"));
			//this.effects_selected = ImageIO.read(new File("img/layout/effects_effect_border_selectedbig.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		createUI();
	}

	/**
	 * Creates UI for this panel.
	 */
	private void createUI() {

		setLayout(new BorderLayout());
		setPreferredSize(new Dimension(600, 130));
		setMinimumSize(new Dimension(600, 110));
		setMaximumSize(new Dimension(600, 130));
		setBackground(new Color(68, 66, 66));
		
		effectsPanel = new JPanel();
		
		effectsPanel.setLayout(new GridBagLayout());
		effectsPanel.setBackground(new Color(68, 66, 66));
		
		scrollPane = new JScrollPane(effectsPanel,ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setBackground(new Color(68, 66, 66));
		
		this.add(scrollPane, BorderLayout.CENTER);
		
		createThumbnail("");
	}

	/**
	 * Creates effect buttons dynamically.
	 * @param effectFactory
	 */
	public void createButton(EffectFactory effectFactory) { 
		GridBagConstraints gbc = new GridBagConstraints();
		int posx = 1;

		for (int i = 0; i <= effectFactory.getEffects().size() - 1; i++) {
			JButton button = new JButton();
			button.setIcon(new ImageIcon(effectFactory.getEffects().get(i)
					.getThumb()));
			button.setOpaque(false);
			button.setFocusPainted(false);
			button.setBorderPainted(false);
			button.setContentAreaFilled(false);
			button.addActionListener(this.uiListener);
			button.setActionCommand(effectFactory.getEffects().get(i)
					.getDisplayName());

			JLayeredPane border = new JLayeredPane();
			border.setLayout(new GridBagLayout());

			gbc.gridx = posx;
			gbc.gridy = 1;
			gbc.gridheight = 1;
			gbc.gridwidth = 1;

			ImagePanel imagePanel = new ImagePanel(imagePanelBG);
					//"img/layout/effects_effect_borderbig.png");
			imagePanel.setPreferredSize(new Dimension(140, 82));
			imagePanel.setMinimumSize(new Dimension(140, 82));
			imagePanel.setMaximumSize(new Dimension(140, 82));
			imagelist.add(imagePanel);

			ImagePanel selectedPanel = new ImagePanel(selectedPanelBG);
					//"img/layout/effects_effect_border_selectedbig.png");
			selectedPanel.setPreferredSize(new Dimension(140, 82));
			selectedPanel.setMinimumSize(new Dimension(140, 82));
			selectedPanel.setMaximumSize(new Dimension(140, 82));
			selectedPanel.setVisible(false);
			selectedborderlist.add(selectedPanel);

			gbc.insets = new Insets(3, 0, 0, 0);
			border.add(imagePanel, gbc); 
			gbc.insets = new Insets(8, 0, 0, 0);
			border.add(selectedPanel, gbc);

			gbc.insets = new Insets(0, 0, 0, 0);
			border.add(button, gbc); 

			JLabel text = new JLabel(effectFactory.getEffects().get(i)
					.getDisplayName());

			gbc.gridx = posx;
			gbc.gridy = 2;
			gbc.insets = new Insets(0, 0, 0, 0);
			border.add(text, gbc);

			validate();
			repaint();

			gbc.gridx = posx;
			gbc.gridy = 1;
			effectsPanel.add(border, gbc);
			
			effectsPanel.revalidate();
			scrollPane.revalidate();
			validate();
			repaint();

			posx++;
		}

	}

	/**
	 * Creates thumbnails. There are two options to do so.
	 * First, create thumbnails from an image-file.
	 * Second, create thumbnails from media file. This is done
	 * by taking a snapshot.
	 * @param path
	 */
	public void createThumbnail(String path) {
		//thumbFactory = new ThumbnailFactory(this.player,path, 130);
		//BufferedImage image = thumbFactory.getSnapshot();
		Image img = null;
		//img = ImageIO.read(new File("img/icons/effect.png"));
		try {
			img = ImageIO.read(getClass().getResource("/img/icons/effect.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//ImageSize: 130x70!!
		BufferedImage image = null;
	
		//get original thumbnail
		for(int i = 0; i < effectFactory.getEffects().size(); i++) {
			image = bufferImage(img);
			effectFactory.getEffects().get(i).setThumb(effectFactory.getEffects().get(i).applyEffectToBufferedImage(image));
		}
		createButton(effectFactory);
	}
	
	/**
	 * Creates a BufferedImage out of an Image.
	 * @param Image image
	 * @return bufferedImage
	 */
	public static BufferedImage bufferImage(Image image) {
        BufferedImage bufferedImage = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        Graphics2D g = bufferedImage.createGraphics();
        g.drawImage(image, null, null);
        return bufferedImage;
    }

	/**
	 * Returns the effectFactory.
	 * @return effectFactory
	 */
	public EffectFactory getEffectFactory() {
		return this.effectFactory;
	}

	/**
	 * Returns the selected image.
	 * @return effects_selected
	 */
	public BufferedImage getSelectedImage() {
		return this.effects_selected;
	}

	/**
	 * Returns the imageList.
	 * @return imagelist
	 */
	public List<ImagePanel> getImagePanel() {
		return this.imagelist;
	}

	/**
	 * Returns the Borderlist.
	 * @return selectedborderlist
	 */
	public List<ImagePanel> getSelectionList() {
		return this.selectedborderlist;
	}


}