package view;

import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.image.BufferedImage;

import javax.swing.SwingUtilities;

import model.Timeline;
import player.Player;
import uk.co.caprica.vlcj.binding.internal.libvlc_media_t;
import uk.co.caprica.vlcj.logger.Logger;
import uk.co.caprica.vlcj.player.MediaDetails;
import uk.co.caprica.vlcj.player.MediaMeta;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.MediaPlayerEventAdapter;

/**
 * PlayerListener.
 * @author Johann Houszka, Matthias Eigner
 *
 */
public class PlayerListener implements AWTEventListener{
	
	private PlayerMainPanel mainPanel;
	private Player player;
	private PlayerEventListener eventListener;
	private PlayerMouseListener mouseListener;
	private PlayerKeyListener keyListener;
	
	/**
	 * Constructor of this class.
	 * @param PlayerMainPanel mainPanel
	 * @param Player player
	 */
	public PlayerListener(PlayerMainPanel mainPanel, Player player) {
		this.mainPanel = mainPanel;
		this.player = player;
		this.eventListener = new PlayerEventListener();
		this.mouseListener = new PlayerMouseListener();
		this.keyListener = new PlayerKeyListener();
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.AWTEventListener#eventDispatched(java.awt.AWTEvent)
	 */
	public void eventDispatched(AWTEvent event) {
       /*  if(event instanceof KeyEvent) {
             KeyEvent keyEvent = (KeyEvent)event;
             if(keyEvent.getID() == KeyEvent.KEY_PRESSED) {
                 if(keyEvent.getKeyCode() == KeyEvent.VK_F12) {
                     mainPanel.getMainFrame().getJMenuBar().setVisible(! mainPanel.getMainFrame().getJMenuBar().isVisible());
                     mainPanel.getMainFrame().invalidate();
                     mainPanel.getMainFrame().validate();
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_A) {
                     player.getMediaPlayer().setAudioDelay(player.getMediaPlayer().getAudioDelay() - 50000);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_S) {
                	 player.getMediaPlayer().setAudioDelay(player.getMediaPlayer().getAudioDelay() + 50000);
                 }
                  else if(keyEvent.getKeyCode() == KeyEvent.VK_N) {
                	  player.getMediaPlayer().nextFrame();
                  }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_1) {
                	 player.getMediaPlayer().setTime(60000 * 1);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_2) {
                	 player.getMediaPlayer().setTime(60000 * 2);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_3) {
                	 player.getMediaPlayer().setTime(60000 * 3);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_4) {
                	 player.getMediaPlayer().setTime(60000 * 4);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_5) {
                	 player.getMediaPlayer().setTime(60000 * 5);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_6) {
                	 player.getMediaPlayer().setTime(60000 * 6);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_7) {
                	 player.getMediaPlayer().setTime(60000 * 7);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_8) {
                	 player.getMediaPlayer().setTime(60000 * 8);
                 }
                 else if(keyEvent.getKeyCode() == KeyEvent.VK_9) {
                	 player.getMediaPlayer().setTime(60000 * 9);
                 }
             }
         }*/
     }
	
	/**
	 * PlayerEventListener class.
	 * @author Johann Houszka, Matthias Eigner
	 *
	 */
	private final class PlayerEventListener extends MediaPlayerEventAdapter {
        @Override
        public void mediaChanged(MediaPlayer mediaPlayer, libvlc_media_t media, String mrl) {
            Logger.debug("mediaChanged(mediaPlayer={},media={},mrl={})", mediaPlayer, media, mrl);
        }
        @Override
        public void finished(MediaPlayer mediaPlayer) {
            Logger.debug("finished(mediaPlayer={})", mediaPlayer);
        }
        @Override
        public void paused(MediaPlayer mediaPlayer) {
            Logger.debug("paused(mediaPlayer={})", mediaPlayer);
        }
        @Override
        public void playing(MediaPlayer mediaPlayer) {
            Logger.debug("playing(mediaPlayer={})", mediaPlayer);
            try {
            	MediaDetails mediaDetails = mediaPlayer.getMediaDetails();
            	Logger.info("mediaDetails={}", mediaDetails);
            }
            catch (Exception e) {
            	System.out.println("MediaDetailExecption::Cant get media details because video is not playing.");
            }
        }
        @Override
        public void stopped(MediaPlayer mediaPlayer) {
            Logger.debug("stopped(mediaPlayer={})", mediaPlayer);
        }
        @Override
        public void videoOutput(MediaPlayer mediaPlayer, int newCount) {
            Logger.debug("videoOutput(mediaPlayer={},newCount={})", mediaPlayer, newCount);
            if(newCount == 0) {
                return;
            }
            
            MediaDetails mediaDetails = mediaPlayer.getMediaDetails();
            Logger.info("mediaDetails={}", mediaDetails);

            MediaMeta mediaMeta = mediaPlayer.getMediaMeta();
            Logger.info("mediaMeta={}", mediaMeta);

            final Dimension dimension = mediaPlayer.getVideoDimension();
            Logger.debug("dimension={}", dimension);
            if(dimension != null) {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                    	mainPanel.getVideoSurface().setSize(dimension);
                        mainPanel.getMainFrame().pack();
                    }
                });
            }

            // You can set a logo like this if you like...
            /*File logoFile = new File("img/layout/playing_background.png");
            if(logoFile.exists()) {
                mediaPlayer.setLogoFile(logoFile.getAbsolutePath());
                mediaPlayer.setLogoOpacity(0.5f);
                mediaPlayer.setLogoLocation(10, 10);
                mediaPlayer.enableLogo(true);
            }*/

            // Demo the marquee
            /*mediaPlayer.setMarqueeText("vlcj java bindings for vlc");
            mediaPlayer.setMarqueeSize(40);
            mediaPlayer.setMarqueeOpacity(95);
            mediaPlayer.setMarqueeColour(Color.white);
            mediaPlayer.setMarqueeTimeout(5000);
            mediaPlayer.setMarqueeLocation(50, 120);
            mediaPlayer.enableMarquee(true);*/
        }

        @Override
        public void error(MediaPlayer mediaPlayer) {
            Logger.debug("error(mediaPlayer={})", mediaPlayer);
        }

        @Override
        public void mediaSubItemAdded(MediaPlayer mediaPlayer, libvlc_media_t subItem) {
            Logger.debug("mediaSubItemAdded(mediaPlayer={},subItem={})", mediaPlayer, subItem);
        }

        @Override
        public void mediaDurationChanged(MediaPlayer mediaPlayer, long newDuration) {
            Logger.debug("mediaDurationChanged(mediaPlayer={},newDuration={})", mediaPlayer, newDuration);
        }

        @Override
        public void mediaParsedChanged(MediaPlayer mediaPlayer, int newStatus) {
            Logger.debug("mediaParsedChanged(mediaPlayer={},newStatus={})", mediaPlayer, newStatus);
        }

        @Override
        public void mediaFreed(MediaPlayer mediaPlayer) {
            Logger.debug("mediaFreed(mediaPlayer={})", mediaPlayer);
        }

        @Override
        public void mediaStateChanged(MediaPlayer mediaPlayer, int newState) {
            Logger.debug("mediaStateChanged(mediaPlayer={},newState={})", mediaPlayer, newState);
        }

        @Override
        public void mediaMetaChanged(MediaPlayer mediaPlayer, int metaType) {
            Logger.debug("mediaMetaChanged(mediaPlayer={},metaType={})", mediaPlayer, metaType);
        }
    }


    /**
     * Enable Mouse.
     * @param enable
     */
    @SuppressWarnings("unused")
    private void enableMousePointer(boolean enable) {
        Logger.debug("enableMousePointer(enable={})", enable);
        if(enable) {
        	mainPanel.getVideoSurface().setCursor(null);
        }
        else {
            Image blankImage = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
            mainPanel.getVideoSurface().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(blankImage, new Point(0, 0), ""));
        }
    }
    
    /**
     * Broken out position setting, handles updating mediaPlayer
     */
    private void setSliderBasedPosition(Timeline timeline) {
        if(!player.getMediaPlayer().isSeekable()) {
            return;
        }
        float positionValue = (float) timeline.getValue() / (float) Timeline.maxResolution;
        // Avoid end of file freeze-up
        if(positionValue > 0.99f) {
            positionValue = 0.99f;
        }
        player.getMediaPlayer().setPosition(positionValue);
    }
	
	/**
     * PlayerMouseListener class.
     */
   private final class PlayerMouseListener extends MouseAdapter {
       @Override
       public void mouseMoved(MouseEvent e) {
           Logger.trace("mouseMoved(e={})", e);
       }

       @Override
       public void mouseWheelMoved(MouseWheelEvent e) {
           Logger.debug("mouseWheelMoved(e={})", e);
       }

       @Override
       public void mouseEntered(MouseEvent e) {
           Logger.debug("mouseEntered(e={})", e);
       }

       @Override
       public void mouseExited(MouseEvent e) {
           Logger.debug("mouseExited(e={})", e);
       }
       
       private boolean mousePressedPlaying;
       
       @Override
       public void mousePressed(MouseEvent e) {
    	   if(e.getSource() instanceof Timeline) 
    	   {
	           Timeline t = (Timeline) e.getSource(); 
	           t.setValue(e.getX());
	           setSliderBasedPosition((Timeline) e.getSource());
	       }
       }

       @Override
       public void mouseReleased(MouseEvent e) {
    	   if(e.getSource() instanceof Timeline) 
    	   {
    		   Timeline t = (Timeline) e.getSource();
	           t.setValue(e.getX());
	           setSliderBasedPosition((Timeline) e.getSource());
    		   mainPanel.updateUIState(mousePressedPlaying);
    	   }
       }
   }

   /**
    * PlayerKeyListener class.
    */
   private final class PlayerKeyListener extends KeyAdapter {

       @Override
       public void keyPressed(KeyEvent e) {
           Logger.debug("keyPressed(e={})", e);
       }

       @Override
       public void keyReleased(KeyEvent e) {
           Logger.debug("keyReleased(e={})", e);
       }

       @Override
       public void keyTyped(KeyEvent e) {
           Logger.debug("keyTyped(e={})", e);
       }
   }

   
	/**
	 * Returns the eventListener.
	 * @return eventListener
	 */
	public PlayerEventListener getEventListener() {
		return eventListener;
	}
	
	/**
	 * Sets the eventListener.
	 * @param eventListener
	 */
	public void setEventListener(PlayerEventListener eventListener) {
		this.eventListener = eventListener;
	}
	
	/**
	 * Returns the mouseListener.
	 * @return mouseListener
	 */
	public PlayerMouseListener getMouseListener() {
		return mouseListener;
	}
	
	/**
	 * Sets the mouseListener.
	 * @param mouseListener
	 */
	public void setMouseListener(PlayerMouseListener mouseListener) {
		this.mouseListener = mouseListener;
	}
	
	/**
	 * Returns the keyListener.
	 * @return key Listener
	 */
	public PlayerKeyListener getKeyListener() {
		return keyListener;
	}
	
	/**
	 * Sets the keyListener.
	 * @param PlayerKeyListener keyListener
	 */
	public void setKeyListener(PlayerKeyListener keyListener) {
		this.keyListener = keyListener;
	}
	   
}
