package view;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.UIManager.LookAndFeelInfo;

import model.Timeline;
import player.Player;
import save.ConvertVideo;
import uk.co.caprica.vlcj.logger.Logger;
import uk.co.caprica.vlcj.medialist.MediaList;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.direct.DirectMediaPlayer;
import uk.co.caprica.vlcj.player.direct.RenderCallbackAdapter;
import uk.co.caprica.vlcj.player.list.MediaListPlayer;
import uk.co.caprica.vlcj.runtime.RuntimeUtil;
import uk.co.caprica.vlcj.runtime.windows.WindowsCanvas;
import controller.PlayerUIListener;

/**
 * Main Player Panel class. 
 * @author Johann Houszka | 0625523, Matthias Eigner |0625203
 *
 */

public class PlayerMainPanel {

	private JFrame mainFrame;
	
	private PlayerListener listener;
	private PlayerUIListener uiListener;
	
	private Canvas videoSurface;
	private final Player player;
	
	private PlayerControlsPanel controlsPanel;
	private PlayerEffectsPanel effectsPanel;
	private PlayerSettingsPanel settingsPanel;
	private PlayerTimelinePanel timelinePanel;
	private JPanel mainPanel, reloadPanel;
	private JButton reload;
	private BufferedImage image;
	private ImagePane imagePane;
	private MediaList mediaList;
	private MediaListPlayer mediaListPlayer;
	private JMenuItem save, loadSub, createSub;
	
	private long movieDuration;
	
	private boolean loaded;
	
	private final int height = 480;
	private final int width = 720;

    private final ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();

    /**
     * Constructor of this class.
     * This is the main class of the application.
     * Used to build the whole interface.
     * @param Player player
     */
    public PlayerMainPanel(final Player player){
    	this.player = player;
    	
        // Set System L&F
		//LookAndFeelInfo[] landf = UIManager.getInstalledLookAndFeels();
		/*for (int i = 0; i < landf.length; i++) {
			System.out.println("Look & Feel Info: " + landf[i]);
		}*/
		try {
			//UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			//UIManager.setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
	    	//UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
			//UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (ClassNotFoundException e1) {
				e1.printStackTrace();
			} catch (InstantiationException e1) {
				e1.printStackTrace();
			} catch (IllegalAccessException e1) {
				e1.printStackTrace();
			} catch (UnsupportedLookAndFeelException e1) {
				e1.printStackTrace();
			}
		}
    	
	       
    	image = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(width, height);
        image.setAccelerationPriority(1.0f);
        
    	this.movieDuration = 0;

    	listener = new PlayerListener(this, player);
		uiListener = new PlayerUIListener(player, this);
				
		this.setLoaded(false);
    	
    	//reloadButton
    	reload = new JButton();
    	reload.setIcon(new ImageIcon("img/icons/video_load.png"));
    	reload.setPreferredSize(new Dimension(160,160));
    	reload.setMaximumSize(new Dimension(160,160));
    	reload.setBorder(BorderFactory.createEmptyBorder());
    	reload.setBorderPainted(false); 
    	reload.setContentAreaFilled(false); 
    	reload.setFocusPainted(false); 
    	reload.setOpaque(false);
    	// What button looks like normally after setSelected( true ).
    	reload.setSelectedIcon (new ImageIcon("img/icons/video_load_active.png"));
		// what button looks like pressed.
    	reload.setPressedIcon(new ImageIcon("img/icons/video_load_active.png"));
		// what button looks like when mouse hovers over it.
    	reload.setRolloverIcon(new ImageIcon("img/icons/video_load_active.png"));
		// what button looks like when mouse hovers over it after setSelected( true ).
    	reload.setRolloverSelectedIcon(new ImageIcon("img/icons/video_load_active.png"));
          	
    	//reloadPanel
    	reloadPanel = new JPanel();
    	reloadPanel.setOpaque(true);
    	reloadPanel.setBorder(BorderFactory.createEmptyBorder());
    	reloadPanel.setFocusable(false);
    	reloadPanel.setBackground(Color.black);
    	reloadPanel.setLayout(new BorderLayout());
    	reloadPanel.setPreferredSize(new Dimension(200,200));
    	reloadPanel.setMaximumSize(new Dimension(200,200));
    	reloadPanel.add(BorderLayout.CENTER,reload);
    	
		if(RuntimeUtil.isWindows()) {
            // If running on Windows and you want the mouse/keyboard event hack...
            videoSurface = new WindowsCanvas();
        }
        else {
            videoSurface = new Canvas();
        }
		
		Logger.debug("videoSurface={}", videoSurface);
		
		//create and add listeners if you want mouse/key action on video
        //videoSurface.addMouseListener(listener.getMouseListener());
        //videoSurface.addMouseMotionListener(listener.getMouseListener());
        //videoSurface.addMouseWheelListener(listener.getMouseListener());
        //videoSurface.addKeyListener(listener.getKeyListener());
        //videoSurface.setPreferredSize(new Dimension(600,400));
		imagePane = new ImagePane(image);
		imagePane.setPreferredSize(new Dimension(width,height));
        
        reload.addActionListener(uiListener);
    	reload.setActionCommand("reload");

        mainFrame = new JFrame("Title Editor");

        DirectMediaPlayer mediaPlayer = player.getMediaPlayerFactory().newDirectMediaPlayer(width, height, new MainRenderCallback());
        player.setMediaPlayer(mediaPlayer);
         
        mediaListPlayer = player.getMediaPlayerFactory().newMediaListPlayer();
        // Important: associate the media player with the media list player
        mediaListPlayer.setMediaPlayer(mediaPlayer);
        
        mediaList = player.getMediaPlayerFactory().newMediaList();
        mediaListPlayer.setMediaList(mediaList);

        //player.getMediaPlayer().setEnableKeyInputHandling(false);
        //player.getMediaPlayer().setEnableMouseInputHandling(false);
        
        
        //create panels
        effectsPanel = new PlayerEffectsPanel(player, this);
        controlsPanel = new PlayerControlsPanel(player, this);
        settingsPanel = new PlayerSettingsPanel(player, this);
        timelinePanel = new PlayerTimelinePanel(player, this);
        
        //add panels to mainpanel
        mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(BorderLayout.NORTH, effectsPanel);
        mainPanel.add(BorderLayout.CENTER, imagePane);
        mainPanel.add(BorderLayout.CENTER, reloadPanel);
        mainPanel.add(BorderLayout.SOUTH, controlsPanel);
        mainPanel.setPreferredSize(new Dimension(720,500));
        
        //System.out.println(mainPanel.getWidth() + "::" + mainPanel.getHeight());
        
        mainFrame.setLayout(new BorderLayout());
        mainFrame.setBackground(Color.black);
        mainFrame.setMinimumSize(new Dimension(850,685));
        mainFrame.setResizable(false);
        
        //add panels to mainframe
        mainFrame.add(mainPanel, BorderLayout.CENTER);
        mainFrame.add(settingsPanel, BorderLayout.EAST);
        mainFrame.add(timelinePanel, BorderLayout.SOUTH);
        mainFrame.setJMenuBar(buildMenuBar());
        
        mainFrame.pack();
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mainFrame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent evt) {
                Logger.debug("windowClosing(evt={})", evt);

                if(videoSurface instanceof WindowsCanvas) {
                    ((WindowsCanvas)videoSurface).release();
                }

                if(player.getMediaPlayer() != null) {
                	player.getMediaPlayer().release();
                	player.setMediaPlayer(null);
                }

                if(player.getMediaPlayerFactory() != null) {
                	player.getMediaPlayerFactory().release();
                	player.setMediaPlayerFactory(null);
                }
            }
        });
        
        mainFrame.setVisible(true);

        player.getMediaPlayer().addMediaPlayerEventListener(listener.getEventListener());
        
        // Global AWT key handler, you're better off using Swing's InputMap and
        // ActionMap with a JFrame - that would solve all sorts of focus issues too
        Toolkit.getDefaultToolkit().addAWTEventListener(listener, AWTEvent.KEY_EVENT_MASK);
        executorService.scheduleAtFixedRate(new UpdateRunnable(player.getMediaPlayer()), 0L, 1L, TimeUnit.SECONDS);
	}
    
    /**
     * Returns the movie duration.
     * @return movieDuration
     */
    public long getMovieDuration() {
    	return this.movieDuration;
    }
    
    /**
     * Sets the movie duration.
     * @param long dur
     */
    public void setMovieDuration(long dur) {
    	this.movieDuration = dur;
    }
   
    /**
     * Creates menubar for the application.
     * @return JMenubar menubar
     */
    private JMenuBar buildMenuBar() {
    	JMenuBar menuBar = new JMenuBar();

        JMenu file = new JMenu("File");
        JMenu options = new JMenu("Options");
        JMenu help = new JMenu("Help");
        
        JMenuItem open = new JMenuItem("Open File");
        save = new JMenuItem("Save");
        JMenuItem exit = new JMenuItem("Exit");
        JMenuItem about = new JMenuItem("About");
        loadSub = new JMenuItem("Load Subtitles");
        createSub = new JMenuItem("Create new Subtitles");
        //JMenuItem changePath = new JMenuItem("Change Path");
        
        file.setMnemonic('f');
        open.setMnemonic('o');
        save.setMnemonic('s');
        about.setMnemonic('a');
        
        open.setActionCommand("menu_open");
		open.addActionListener(uiListener);
		save.setActionCommand("menu_save");
		save.addActionListener(uiListener);
		save.setEnabled(false);
		exit.setActionCommand("menu_exit");
		exit.addActionListener(uiListener);
		about.setActionCommand("menu_about");
		about.addActionListener(uiListener);
        
        file.add(open);
        file.add(save);
        file.add(exit);
        
        options.add(loadSub);
        loadSub.addActionListener(uiListener);
        loadSub.setActionCommand("menu_loadSub");
        loadSub.setEnabled(false);
        options.add(createSub);
        createSub.addActionListener(uiListener);
        createSub.setActionCommand("menu_createSub");
        createSub.setEnabled(false);
       // options.add(changePath);
       // changePath.addActionListener(uiListener);
       // changePath.setActionCommand("menu_changePath");
        
        help.add(about);
        
        menuBar.add(file);
        menuBar.add(options);
        menuBar.add(help);
        
        return menuBar;
    }

    /*
	 ************************************************************
	 ************************************************************
	 * Getter and Setter Methods used in this class
	 ************************************************************ 
	 ************************************************************
	 */
    
    /**
     * Set the menuItems enabled if 
     * a video is loaded.
     */
    public void setMenuItemsEnabled(boolean status) {
    	this.save.setEnabled(status);
    	this.createSub.setEnabled(status);
    	this.loadSub.setEnabled(status);
    }
    
    /**
     * Add ImagePane and remove the Play-Icon
     */
    public void addImagePane() {
    	mainPanel.remove(reloadPanel); 
    	mainPanel.validate();
    	mainPanel.repaint();
        mainPanel.add(BorderLayout.CENTER, imagePane); 
        mainPanel.validate();
        mainPanel.repaint();
        mainFrame.validate();
        mainFrame.repaint();
    }
    
    /**
     * Remove and add ImagePane if second video is loaded
     */
    public void readdImagePane() {
    	mainPanel.remove(imagePane); 
    	mainPanel.validate();
    	mainPanel.repaint();
        mainPanel.add(BorderLayout.CENTER, imagePane); 
        mainPanel.validate();
        mainPanel.repaint();
        mainFrame.validate();
        mainFrame.repaint();
    }
    
    /**
     * Adds the reload panel.
     */
    public void addReloadPanel() {
    	mainPanel.remove(imagePane); 
    	mainPanel.validate();
    	mainPanel.repaint();
        mainPanel.add(BorderLayout.CENTER, reloadPanel);
        mainPanel.validate();
        mainPanel.repaint();	
        mainFrame.validate();
        mainFrame.repaint();
    }
    
    
	/**
	 * Return the VideoSurface.
	 * @return videosurface
	 */
	public Canvas getVideoSurface() {
		return videoSurface;
	}

	/**
	 * Return the MainFrame.
	 * @return
	 */
	public JFrame getMainFrame() {
		return mainFrame;
	}

	/**
	 * Set MainFrame.
	 * @param mainFrame
	 */
	public void setMainFrame(JFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	/**
	 * Set video surface. Needed to display video.
	 * @param videoSurface
	 */
	public void setVideoSurface(Canvas videoSurface) {
		this.videoSurface = videoSurface;
	}
	
	/**
	 * Returns the Timeline.
	 * @return timeline.
	 */
	public Timeline getTimeline() {
		return this.timelinePanel.getTimeline();
	}
	
	/**
	 * Returns the PlayerListener Object.
	 * @return listener
	 */
	public PlayerListener getListener() { return this.listener; }
	
	/**
	 * Returns the UIListener Object.
	 * @return uiListener
	 */
	public PlayerUIListener getUIListener() { return this.uiListener; }
	
	/**
	 * Update Timeline-Position if play is clicked.
	 * @param mousePressedPlaying
	 */
	public void updateUIState(boolean mousePressedPlaying) {
		if(this.isLoaded())
		{
	        updatePosition(player.getMediaPlayer().getPosition() * (float) Timeline.maxResolution);
		}
	}
	
	/**
	 * Updates position of Timeline and displays Time.
	 * @param value
	 */
	public void updatePosition(float value) {
		 this.timelinePanel.getTimeline().setValueFloat(value);
		 String timeText = this.controlsPanel.calcTime();
		 this.controlsPanel.getTime().setText(timeText);
    }
	
	/**
	 * Returns if video is loaded or not.
	 * @return
	 */
	public boolean isLoaded() {
		return loaded;
	}

	/**
	 * Sets status of video to loaded.
	 * @param loaded
	 */
	public void setLoaded(boolean loaded) {
		this.loaded = loaded;
	}

	/**
	 * adds String to mediaFile to mediaList.
	 * @param media
	 */
	public void addToMediaList(String media) {
		mediaListPlayer.getMediaList().addMedia(media);
        System.out.println(mediaListPlayer.getMediaList().items());
	}
	
	/**
	 * Returns the ControlsPanel.
	 * @return controlsPanel
	 */
	public PlayerControlsPanel getControlsPanel() {
		return controlsPanel;
	}

	/**
	 * Returns the EffectsPanel.
	 * @return effectsPanel
	 */
	public PlayerEffectsPanel getEffectsPanel() {
		return effectsPanel;
	}
	
	/**
	 * Returns the SettingsPanel.
	 * @return settingsPanel
	 */
	public PlayerSettingsPanel getSettingsPanel() {
		return settingsPanel;
	}

	/**
	 * Used to update the Timeline.
	 * @author Matthias Eigner
	 *
	 */
	private final class UpdateRunnable implements Runnable {

        private final MediaPlayer mediaPlayer;

        private UpdateRunnable(MediaPlayer mediaPlayer) {
            this.mediaPlayer = mediaPlayer;
        }

        @Override
        public void run() {
            final float position = (mediaPlayer.getPosition()* Timeline.maxResolution);
            final float pos = mediaPlayer.getPosition();
            // Updates to user interface components must be executed on the Event
            // Dispatch Thread
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    if(mediaPlayer.isPlaying()) {
                        updatePosition(position);
                    }
                    if(pos >= 1) {
                    	//player.getMediaPlayer().stop();
                    	setVideoToStart();
                    }
                }
            });
        }
    }
	
	/**
	 * Sets video to start position.
	 */
	public void setVideoToStart() {
		//mediaListPlayer.stop();
		player.getMediaPlayer().play();
		player.getMediaPlayer().pause();
		player.getMediaPlayer().setPosition(0.0f);
	
    	controlsPanel.setPaused();
    	updatePosition(0);
	}
	
	/**
	 * Returns the MediaListPlayer.
	 * @return
	 */
	public MediaListPlayer getMediaListPlayer() {
		return this.mediaListPlayer;
	}
	
	/**
	 * Returns the player.
	 * @return
	 */
	public Player getPlayer() {
		return this.player;
	}
	
	 /**
	  * Private class used only in the MainPanel. 
	  * @author Matthias Eigner
	  *
	  */
	@SuppressWarnings("serial")
	  private final class ImagePane extends JPanel {

	    private final BufferedImage image;

	    public ImagePane(BufferedImage image) {
	      this.image = image;
	    }

	    @Override
	    public void paint(Graphics g) {
	      Graphics2D g2 = (Graphics2D)g;
	      g2.drawImage(image, null, 0, 0);
	    }
	  }

	/**
	 * Private class used in the MainPanel. 
	 * Here, all the effects get applied to to video frames.
	 * @author Matthias Eigner, Johann Houszka
	 *
	 */
	private final class MainRenderCallback extends RenderCallbackAdapter {
		public MainRenderCallback() {
			super (new int [width * height]);
		}
		
		private Graphics g;
		
		/* (non-Javadoc)
		 * @see uk.co.caprica.vlcj.player.direct.RenderCallbackAdapter#onDisplay(int[])
		 * overlay image with own elements (Title, Images)
		 */
		@Override
		protected void onDisplay(int[] data) {	
			try {
				data = uiListener.getActiveEffect().applyEffect(data);

				// add Graphics
				image.setRGB(0, 0, width, height, data, 0, width);
				g = image.getGraphics();
				
				// this method supports the start and duration property of PlayerImage
				g.drawImage(uiListener.getImages().getImageAtPosition(player.getMediaPlayer().getTime(), image.getWidth(), image.getHeight()), 0, 0, image.getWidth(), image.getHeight(), null);
				
				// add Titles
				g.drawImage(uiListener.getTitles().getImageAtPosition(player.getMediaPlayer().getTime(), image.getWidth(), image.getHeight()), 0, 0, image.getWidth(), image.getHeight(), null);
				imagePane.repaint();
			} catch (NullPointerException ex) {
				System.out.println("NullPointerException::Stream not correctly closed!");
			}
		}
	}

}


