package view;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import player.Player;
import save.SaveVideo;
import controller.PlayerUIListener;

/**
 * Builds the UI for the SaveDialog.
 * @author Johann Houszka 0625523
 * 
 */
public class PlayerSaveDialog implements ActionListener {

	private Player player;
	private PlayerUIListener uiListener;
	private final String[] FORMAT;
	private final String[] SUBTITLE;
	private final String[] SUB_LANGUAGE;
	private PlayerMainPanel mainPanel;
	private JTextField pathField, subField;
	private String inFile;
	private JComboBox formatBox, subBox, langBox;
	private JProgressBar progBar;
	private JCheckBox checkBox;
	    
	/**
	 * Constructor of this class.
	 * @param player
	 * @param mainPanel
	 */
	public PlayerSaveDialog(Player player, PlayerMainPanel mainPanel, String inFile) {
		this.player = player;
		this.uiListener = mainPanel.getUIListener();
		this.mainPanel = mainPanel;
		this.inFile = inFile;

		FORMAT = new String[] { "mkv", "mp4"};
		SUBTITLE = new String[] { "No Subtitles", "Render", "File"};
		SUB_LANGUAGE = new String[] { "German", "English","French", "Italian", "Spanish"};
		
		createUI();
	}

	/**
	 * Creates UI for this Dialog.
	 * Perform operations to build String for savepath.
	 */
	private void createUI() {

		JLabel formatLbl = new JLabel("Video Format");
		formatLbl.setBounds(5, 0, 150, 20);
		formatBox = new JComboBox(FORMAT);
		formatBox.setBounds(0, 20, 150, 30);
		JLabel suboptLbl = new JLabel("Subtitle Options");
		suboptLbl.setBounds(5, 60, 150, 20);
		subBox = new JComboBox(SUBTITLE);
		subBox.setBounds(0, 80, 150, 30);
		subBox.setActionCommand("save_sub");
		subBox.addActionListener(this);
		
		
		JLabel subLang = new JLabel("Subtitle Language");
		subLang.setBounds(5, 120, 150, 20);
		langBox = new JComboBox(SUB_LANGUAGE);
		langBox.setBounds(0, 140, 150, 30);
		langBox.setEnabled(false);
		
		
		subField = new JTextField();
		subField.setVisible(false);
		subField.setBounds(32, 234, 150, 30);
		JButton pathBtnsub = new JButton("...");
		pathBtnsub.setVisible(false);
		pathBtnsub.setActionCommand("sub_file");
		pathBtnsub.addActionListener(this);
		pathBtnsub.setBounds(192, 234, 50, 30);
		JLabel subLbl = new JLabel("Path to Subtitle-File:");
		subLbl.setVisible(false);
		subLbl.setBounds(37, 214, 150, 20);
		
		JLabel checkLabel = new JLabel("Only Subtitles");
		checkLabel.setBounds(170,60, 100, 20);
		checkBox = new JCheckBox();
		checkBox.setBounds(170,80, 30, 30);
		checkBox.setToolTipText("No effects/titles/graphics are applied to video");
		
		pathField = new JTextField();
		pathField.setBounds(5, 202, 150, 30);
		JButton pathBtn = new JButton("...");
		pathBtn.setActionCommand("save_file");
		pathBtn.addActionListener(this);
		pathBtn.setBounds(165, 202, 50, 30);
		JLabel saveLbl = new JLabel("Save File to:");
		saveLbl.setBounds(10, 182, 150, 20);

		JPanel savePanel = new JPanel();
		savePanel.setLayout(null);
		savePanel.setPreferredSize(new Dimension(260, 282)); 
		savePanel.add(formatLbl);
		savePanel.add(formatBox);
		savePanel.add(suboptLbl);
		savePanel.add(subBox);
		savePanel.add(checkLabel);
		savePanel.add(checkBox);
		savePanel.add(subLbl);
		savePanel.add(subLang);
		savePanel.add(langBox);
		savePanel.add(subField);
		savePanel.add(pathBtnsub);
		savePanel.add(saveLbl);
		savePanel.add(pathField);
		savePanel.add(pathBtn);

		int response = JOptionPane.showConfirmDialog(null, savePanel,
				"Save Options", JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.PLAIN_MESSAGE);

		if (response == JOptionPane.OK_OPTION) {
			System.out.println("Your preferences: "
					+ formatBox.getSelectedItem() + "::"
					+ subBox.getSelectedItem());
	
			if(this.pathField.getText().toString() == "") {
				System.out.println("No output directory found!");
				JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
						"No Output Directory declared. Please set path first.");
			}
			else {
				//outputFile
				String outFile = buildOutPath();
				//subtitle options
				String suboptions = subBox.getSelectedItem().toString();
				//video options
				String videooptions = formatBox.getSelectedItem().toString();
				//subtitleFile
				String subtitleFile = this.subField.getText();
				
				//subtitleLanguage
				//0..german
				//1..englisch
				//2..franzsisch
				//3..italienisch
				//4..spanish
				int subtitleLang = this.langBox.getSelectedIndex();
				boolean check = this.checkBox.isSelected();
				
				System.out.println("SaveDialog - Call SaveVideo-Class (save file with XUGGLER/FFMPEG)");
				SaveVideo saveVideo = new SaveVideo(this.player,this.inFile, outFile, suboptions, videooptions, subtitleFile, subtitleLang, check, this.uiListener, this.mainPanel);
				
				uiListener.setSavingStatus(true);
			}
			
		}
		if (response == JOptionPane.CANCEL_OPTION) {
			System.out.println("Cancel Saving");
		}
	}

	private String buildOutPath() { 
		StringBuilder sb = new StringBuilder(100);
		
		//get file ending from formatBox
		String ending = formatBox.getSelectedItem().toString();
		
		String[] path = this.pathField.getText().split("/");
		String[] pathname =  path[path.length-1].split("\\."); //get file-ending from SaveDialog
		
		//autocorrection if false file-ending was declared 
		if(pathname.length > 1 && !pathname[1].toLowerCase().equals(ending.toLowerCase())) {
			System.out.println("Wrong FileEnding.");
			for(int i = 0; i < path.length-1; i++) {
				sb.append(path[i]+"/");
			}
			String[] filename = path[path.length-1].split("\\.");
			sb.append(filename[0]);
			System.out.println(filename[0]);
			sb.append("." + ending.toLowerCase());
		}
		else if (pathname.length <= 1) {
			System.out.println("No FileEnding.");
			sb.append(pathField.getText().toString()+"."+ending.toLowerCase());
		}
		else {
			System.out.println("Correct FileEnding.");
			sb.append(pathField.getText().toString());
		}
		System.out.println("Output-Path: " + sb.toString());
		return sb.toString();
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("save_file")) {

			System.out.println("UIListener:command= Path to OutputFile");
			JFileChooser fc = new JFileChooser();
			FileFilter filter1 = new ExtensionFileFilter("Video Formats (.mkv, .mp4)", new String[] { "mkv", "mp4" });
		    fc.setFileFilter(filter1);
			int returnVal = fc.showSaveDialog(null);
			File f;
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				f = fc.getSelectedFile();
				System.out.println(f.getAbsolutePath());
				setPath(f.getAbsolutePath());
			}
		}
		else if (e.getActionCommand().equals("sub_file")) {

			System.out.println("UIListener:command= Path to SubtitleFile");
			JFileChooser fc = new JFileChooser();
			FileFilter filter1 = new ExtensionFileFilter("Subtitle File (.srt, .ass)", new String[] { "srt", "ass" });
		    fc.setFileFilter(filter1);
			int returnVal = fc.showSaveDialog(null);
			File f;
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				f = fc.getSelectedFile();
				System.out.println(f.getAbsolutePath());
				setSubPath(f.getAbsolutePath());
			}
		}
		else if (e.getActionCommand().equals("save_sub")) {
			if(!this.subBox.getSelectedItem().toString().equals("No Subtitles")) {
				this.langBox.setEnabled(true);
			}
			else {
				this.langBox.setEnabled(false);
			}
		}

	}

	/**
	 * Set SavePath to textField.
	 * @param path
	 */
	public void setPath(String path) {
		this.pathField.setText(path);
	}
	
	/**
	 * Set Path to subtitle File.
	 * @param path
	 */
	public void setSubPath(String path) {
		this.subField.setText(path);
	}

	/**
	 * Class to customize JFileChooser FilterOptions. 
	 * @author Johann Houszka
	 */
	class ExtensionFileFilter extends FileFilter {
		  String description;

		  String extensions[];

		  public ExtensionFileFilter(String description, String extension) {
		    this(description, new String[] { extension });
		  }

		  public ExtensionFileFilter(String description, String extensions[]) {
		    if (description == null) {
		      this.description = extensions[0];
		    } else {
		      this.description = description;
		    }
		    this.extensions = (String[]) extensions.clone();
		    toLower(this.extensions);
		  }

		  private void toLower(String array[]) {
		    for (int i = 0, n = array.length; i < n; i++) {
		      array[i] = array[i].toLowerCase();
		    }
		  }

		  public String getDescription() {
		    return description;
		  }

		  public boolean accept(File file) {
		    if (file.isDirectory()) {
		      return true;
		    } else {
		      String path = file.getAbsolutePath().toLowerCase();
		      for (int i = 0, n = extensions.length; i < n; i++) {
		        String extension = extensions[i];
		        if ((path.endsWith(extension) && (path.charAt(path.length() - extension.length() - 1)) == '.')) {
		          return true;
		        }
		      }
		    }
		    return false;
		  }
	}

}
