package view;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.JColorChooser;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import model.PlayerImage;
import model.PlayerImages.ImageStore;
import model.Subtitle;
import model.Title;
import player.Player;
import controller.ImageCellRenderer;
import controller.PlayerUIListener;

/**
 * Setting Panel Class.
 * Initializes UI for the SettingsPanel.
 * Consists of 3 Parts:
 * 	- TitlePanel
 * 	- SubtitlePanel
 * 	- GraphicsPanel
 * @author Johann Houszka, Matthias Eigner
 * 
 */
public class PlayerSettingsPanel extends JPanel {
	
	/**
	 * Default ID
	 */
	private static final long serialVersionUID = 1L;
	
	//Main variables
	private Player player;
	private PlayerMainPanel mainPanel;
	private PlayerUIListener uiListener;
	private JTabbedPane settingsPane;
	private JComponent titlePanel, subtitlePanel, graphicsPanel;
	private DefaultListModel model_subtitle, model_graphics, model_title;
	private MouseListener mouseListener;
	
	//TitlePanel
	private String text_insert_title = "Insert Title";
	private String text_edit_title = "Save Changes";
	private JTextField title_color;
	private JTextArea title_content;
	private JButton btn_title_action, btnTitleDelete;
	private JSpinner title_start;
	private JComboBox title_font;
	private JSpinner title_duration, title_y, title_x, title_size ;
	private Title active_title;
	private JList titleList;
	private ListSelectionListener myListener_title;
	private JButton color_selection_title;
	private JLabel color_title_preview;
	private Color default_color_title;
		
	//SubtitlePanel
	private String text_insert_subtitle = "Insert Subtitle";
	private String text_edit_subtitle = "Save Changes";
	private JTextArea subtitle_value;
	private JSpinner subtitle_duration, subtitle_position;
	private JButton btn_subtitle_action, btnSubtitleNew, btnSubtitleDelete;
	private JList subtitleList;
	private Subtitle active_subtitle;
	private ListSelectionListener myListener_subtitle;
	private int _currentSub;
	
	//GraphicsPanel
	private JTextField textField;
	private JSpinner image_posx, image_posy, image_duration, image_start;
	private JButton picBtn, btnGraphicDelete, btnImageEinfgen;
	private JList imageList;
	private ArrayList<ImageStore> imageStore;
	private int imageOpacity;
	private JSlider image_opa, image_scale;
	
	
	
	/**
	 * Constructor for PlayerSettingsPanel.
	 * Instantiates this Class.
	 * @param player
	 * @param mainPanel
	 */
	public PlayerSettingsPanel(Player player, PlayerMainPanel mainPanel) {
		this.player = player;
		this.uiListener = mainPanel.getUIListener();
		this.mainPanel = mainPanel;
		this.imageOpacity = 50;
		this.imageStore = new ArrayList<ImageStore>();
		
		createUI();
	}
	
	/**
	 * Creates the UI of the SettingsPanel.
	 */
	private void createUI() {

		/*
		 ************************************************************
		 ************************************************************
		 * Main Settings for the whole Panel.
		 ************************************************************ 
		 ************************************************************
		 */
		setPreferredSize(new Dimension(230, 600));
		setMinimumSize(new Dimension(230,600));
		setBackground(new Color(220, 220, 220));
		setLayout(new BorderLayout());
		
		settingsPane = new JTabbedPane();
		settingsPane.setPreferredSize(new Dimension(200,600));
		settingsPane.setMinimumSize(new Dimension(200,600));
		
		titlePanel = new JPanel();
		titlePanel.setLayout(null);
		titlePanel.setBackground(new Color(220, 220, 220));
		subtitlePanel = new JPanel();
		subtitlePanel.setLayout(null);
		subtitlePanel.setBackground(new Color(220, 220, 220));
		graphicsPanel = new JPanel();
		graphicsPanel.setLayout(null);
		graphicsPanel.setBackground(new Color(220, 220, 220));
		
		settingsPane.addTab("Title", titlePanel);
		settingsPane.addTab("Subtitle", subtitlePanel);
		settingsPane.addTab("Graphics", graphicsPanel);
		add(settingsPane, BorderLayout.CENTER);
		
		
		/*
		 ************************************************************
		 ************************************************************
		 * Title Tab Components
		 ************************************************************ 
		 ************************************************************
		 */
		
		JLabel header1 = new JLabel("Title");
		header1.setBounds(10, 6, 177, 23);
		header1.setFont(new Font("Verdana", Font.PLAIN, 20));
		header1.setHorizontalAlignment(JLabel.LEFT);
		header1.setForeground(new Color(20, 20, 20));
		//header1.setBackground(Color.RED);
		header1.setPreferredSize(new Dimension(100, 50));
		
		titlePanel.add(header1);
		
		JLabel lbl_title_Start = new JLabel("Start:");
		lbl_title_Start.setBounds(11, 214, 59, 16);
		titlePanel.add(lbl_title_Start);
		
		title_color = new JTextField();
		title_color.setVisible(false);
		title_color.setText("#000000");
		title_color.setBounds(55, 140, 22, 28);
		titlePanel.add(title_color);
		title_color.setColumns(10);
		
		JLabel lblFarbe = new JLabel("Color:");
		lblFarbe.setBounds(10, 146, 59, 16);
		titlePanel.add(lblFarbe);
		
		title_font = new JComboBox();
		title_font.setModel(new DefaultComboBoxModel(this.player.getFonts()));
		title_font.setBounds(87, 107, 108, 27);
		titlePanel.add(title_font);
		
		JLabel lblSchriftart = new JLabel("Font:");
		lblSchriftart.setBounds(10, 115, 73, 16);
		titlePanel.add(lblSchriftart);
		
		btn_title_action = new JButton(this.text_insert_title);
		btn_title_action.setBounds(5, 326, 210, 29);
		btn_title_action.addActionListener(uiListener);
		btn_title_action.setActionCommand("titel");
		titlePanel.add(btn_title_action);
		btn_title_action.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				editTitleAction(arg0);
			}
			
		});
		
		title_start = new JSpinner();
		title_start.setBounds(91, 208, 81, 28);
		titlePanel.add(title_start);
		
		title_size = new JSpinner();
		title_size.setBounds(90, 174, 81, 28);
		titlePanel.add(title_size);
		
		JLabel lblSchriftgre = new JLabel("Textsize:");
		//lblSchriftgre.setFont(new Font("Lucida Grande", Font.PLAIN, 12));
		lblSchriftgre.setBounds(10, 180, 100, 16);
		titlePanel.add(lblSchriftgre);
		
		title_content = new JTextArea();
		title_content.setText("");
		title_content.setBounds(10, 41, 177, 54);
		title_content.setBorder(new LineBorder(new Color(150, 150, 150)));
		titlePanel.add(title_content);
		
		JLabel label_1 = new JLabel("ms");
		label_1.setBounds(174, 214, 22, 16);
		titlePanel.add(label_1);
		
		JLabel lblPt = new JLabel("pt");
		lblPt.setBounds(174, 181, 22, 16);
		titlePanel.add(lblPt);
		
		JScrollPane scrollPane_2 = new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane_2.setViewportBorder(new LineBorder(Color.LIGHT_GRAY));
		scrollPane_2.setPreferredSize(new Dimension(190, 600));
		scrollPane_2.setMaximumSize(new Dimension(190, 600));
		scrollPane_2.setBounds(5, 360, 207, 194);
		titlePanel.add(scrollPane_2);
		
		model_title = new DefaultListModel();
		titleList = new JList(model_title);
		scrollPane_2.setViewportView(titleList);
		
		title_duration = new JSpinner();
		title_duration.setBounds(90, 242, 81, 28);
		titlePanel.add(title_duration);
		
		JLabel label_2 = new JLabel("ms");
		label_2.setBounds(173, 248, 22, 16);
		titlePanel.add(label_2);
		
		JLabel l_title_Duration = new JLabel("Duration:");
		l_title_Duration.setBounds(10, 248, 59, 16);
		titlePanel.add(l_title_Duration);
		
		JLabel label_5 = new JLabel("Pos (x, y):");
		label_5.setBounds(12, 288, 73, 16);
		titlePanel.add(label_5);
		
		title_x = new JSpinner();
		title_x.setBounds(82, 282, 60, 28);
		titlePanel.add(title_x);
		
		title_y = new JSpinner();
		title_y.setBounds(147, 282, 60, 28);
		titlePanel.add(title_y);
		
		btnTitleDelete = new JButton("");
		btnTitleDelete.setBounds(182, 10, 16, 16);
		btnTitleDelete.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		btnTitleDelete.setIcon(new ImageIcon("img/icons/cross.png"));
		btnTitleDelete.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				uiListener.getTitles().deleteTitle(active_title);
				fillTitleList(uiListener.getTitles().getTitles());
				newTitle();
			}
			
		});
		titlePanel.add(btnTitleDelete);
		
		default_color_title = new Color(0,0,0);
		
		color_selection_title = new JButton("Choose");
		color_selection_title.setBounds(120, 141, 75, 28); //(110, 141, 85, 28);
		titlePanel.add(color_selection_title);
		color_selection_title.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				Color selectedColor = JColorChooser.showDialog(null, "Pick a Color", default_color_title);
		                
		                if (selectedColor != null)
		                {
		                	default_color_title = selectedColor;
		                	color_title_preview.setBackground(default_color_title);
		                	String rgb = Integer.toHexString(default_color_title.getRGB());
		                	System.out.println(rgb+"-"+rgb.substring(2, rgb.length()));
		                	title_color.setText("#"+rgb.substring(2, rgb.length()));
		                    /*tracker.append("\nThe selected color is make up of Red: " 
		                        + selectedColor.getRed() + " Blue: " + selectedColor.getBlue()
		                        + " Green: " + selectedColor.getGreen());*/
		                }
			}
			
		});
		
		color_title_preview = new JLabel("");
		color_title_preview.setBounds(90, 142, 25, 25); //16,16
		titlePanel.add(color_title_preview);
		color_title_preview.setBackground(default_color_title);
		color_title_preview.setOpaque(true);

		btnTitleDelete.setVisible(false);
		
		/*
		 ************************************************************
		 ************************************************************
		 * Subtitle Tab Components
		 ************************************************************ 
		 ************************************************************
		 */
		
		JLabel lblUntertitel = new JLabel("Subtitles");
		lblUntertitel.setPreferredSize(new Dimension(100, 50));
		lblUntertitel.setHorizontalAlignment(SwingConstants.LEFT);
		lblUntertitel.setForeground(new Color(20, 20, 20));
		lblUntertitel.setFont(new Font("Verdana", Font.PLAIN, 20));
		//lblUntertitel.setBackground(Color.RED);
		lblUntertitel.setBounds(10, 6, 99, 23);
		subtitlePanel.add(lblUntertitel);
		
		btn_subtitle_action = new JButton(this.text_insert_subtitle);
		btn_subtitle_action.setBounds(6, 160, 197, 29);
		//btn_subtitle_action.setBounds(6, 160, 197, 29);
		
		btn_subtitle_action.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				editSubtitleAction(arg0);
			}
			
		});
		subtitlePanel.add(btn_subtitle_action);
		
		JLabel label = new JLabel("Duration:");
		label.setBounds(10, 134, 61, 16);
		subtitlePanel.add(label);
		
		subtitle_duration = new JSpinner();
		subtitle_duration.setBounds(72, 128, 99, 28);
		subtitlePanel.add(subtitle_duration);
		
		JLabel lblSek = new JLabel("ms");
		lblSek.setBounds(173, 134, 22, 16);
		subtitlePanel.add(lblSek);
		
		
		model_subtitle = new DefaultListModel();
		
		JLabel lblPosition = new JLabel("Position:");
		lblPosition.setBounds(10, 107, 61, 16);
		subtitlePanel.add(lblPosition);
		
		subtitle_position = new JSpinner();
		subtitle_position.setBounds(72, 100, 99, 28);
		subtitlePanel.add(subtitle_position);
		
		JLabel label_3 = new JLabel("ms");
		label_3.setBounds(173, 107, 22, 16);
		subtitlePanel.add(label_3);
		JScrollPane scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setViewportBorder(new LineBorder(Color.LIGHT_GRAY));
		scrollPane.setBounds(6, 190, 197, 358);
		scrollPane.setMaximumSize(new Dimension(190,600));
		scrollPane.setPreferredSize(new Dimension(190,600));
		subtitlePanel.add(scrollPane, BorderLayout.CENTER);
		subtitleList = new JList(model_subtitle);
		subtitleList.setBorder(null);
		scrollPane.setViewportView(subtitleList);
		
		this.mouseListener = (new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2){
					System.out.println("Now the video should jump to the position of the subtitle!");
					jumpToSubtitlePosition();
				}
			}
			@Override
			public void mouseEntered(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			@Override
			public void mouseExited(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			@Override
			public void mousePressed(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			@Override
			public void mouseReleased(MouseEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		});
		subtitleList.addMouseListener(mouseListener);
		 
		
		btnSubtitleNew = new JButton("");
		btnSubtitleNew.setBounds(182, 10, 16, 16);
		btnSubtitleNew.setIcon(new ImageIcon("img/icons/add.png"));
		btnSubtitleNew.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		btnSubtitleNew.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				newSubtitle();
			}
			
		});
		subtitlePanel.add(btnSubtitleNew);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(6, 41, 197, 54);
		//scrollPane_1.setBounds(6, 41, 197, 54);
		subtitlePanel.add(scrollPane_1);
		
		subtitle_value = new JTextArea();
		scrollPane_1.setViewportView(subtitle_value);
		subtitle_value.setText("");
		subtitle_value.setBorder(BorderFactory.createLineBorder(new Color(150, 150, 150), 1));
		
		btnSubtitleDelete = new JButton("");
		btnSubtitleDelete.setBounds(182, 10, 16, 16);
		btnSubtitleDelete.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		btnSubtitleDelete.setIcon(new ImageIcon("img/icons/cross.png"));
		btnSubtitleDelete.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				uiListener.getSubtitles().deleteSubTitle(active_subtitle);
				fillSubtitleList(uiListener.getSubtitles().getSubtitles());
				newSubtitle();
			}
			
		});
		subtitlePanel.add(btnSubtitleDelete);
		btnSubtitleDelete.setVisible(false);
		btnSubtitleNew.setVisible(false);		
		
		
		/*
		 ************************************************************
		 ************************************************************
		 * Graphics Tab Components
		 ************************************************************ 
		 ************************************************************
		 */
		
		JLabel lblGraphics = new JLabel("Images");
		lblGraphics.setPreferredSize(new Dimension(100, 50));
		lblGraphics.setHorizontalAlignment(SwingConstants.LEFT);
		lblGraphics.setForeground(new Color(20, 20, 20));
		lblGraphics.setFont(new Font("Verdana", Font.PLAIN, 20));
		//lblGraphics.setBackground(Color.RED);
		lblGraphics.setBounds(10, 6, 177, 23);
		graphicsPanel.add(lblGraphics);
		
		JLabel label_graphics = new JLabel("Duration:");
		label_graphics.setBounds(10, 195, 73, 16);
		graphicsPanel.add(label_graphics);
		
		JLabel lbl_pos = new JLabel("Pos (x, y):");
		lbl_pos.setBounds(10, 226, 73, 16);
		graphicsPanel.add(lbl_pos);
		
		JLabel lbl_scale = new JLabel("Scale:");
		lbl_scale.setBounds(10, 258, 73, 16);
		graphicsPanel.add(lbl_scale);
		
		JLabel lbl_opa = new JLabel("Opacity:");
		lbl_opa.setVisible(false);
		lbl_opa.setBounds(10, 254, 73, 16);
		graphicsPanel.add(lbl_opa);
		
		btnImageEinfgen = new JButton("Insert image");
		//btn_title_action.setBounds(10, 316, 200, 29);
		btnImageEinfgen.setBounds(5, 286, 200, 29); //0,286,210,29
		btnImageEinfgen.addActionListener(uiListener);
		btnImageEinfgen.setActionCommand("graphics_add");
		graphicsPanel.add(btnImageEinfgen);
		
		/*SpinnerNumberModel spinnermodel_dur = new SpinnerNumberModel(0,0,100000,1);
		SpinnerNumberModel spinnermodel_start = new SpinnerNumberModel(0,0,100000,1);
		SpinnerNumberModel spinnerposxmodel = new SpinnerNumberModel(0,0,720,1);
		SpinnerNumberModel spinnerposymodel = new SpinnerNumberModel(0,0,480,1);*/
		
		JLabel lblStart = new JLabel("Start:");
		lblStart.setBounds(10, 160, 73, 16);
		graphicsPanel.add(lblStart);
		
		image_start = new JSpinner();
		//image_start.setModel(spinnermodel_start);
		image_start.setBounds(65, 160, 90, 28);
		graphicsPanel.add(image_start);
		
		JLabel label_4 = new JLabel("ms");
		label_4.setBounds(173, 160, 22, 16);
		graphicsPanel.add(label_4);
		
		image_duration = new JSpinner();
		//image_duration.setModel(spinnermodel_dur);
		image_duration.setBounds(80, 190, 90, 28);
		graphicsPanel.add(image_duration);
		
		image_posx = new JSpinner();
		//image_posx.setModel(spinnerposxmodel);
		image_posx.setBounds(80, 220, 60, 28);
		graphicsPanel.add(image_posx);
		
		image_posy = new JSpinner();
		//image_posy.setModel(spinnerposymodel);
		image_posy.setBounds(145, 220, 60, 28);
		graphicsPanel.add(image_posy);
		
		//create HashTable for Scale-Spinner
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		labelTable.put(0, new JLabel("0%"));
		labelTable.put(100, new JLabel("100%"));
		labelTable.put(400, new JLabel("400%"));
		image_scale = new JSlider(JSlider.HORIZONTAL,0, 400, 100);
		image_scale.setBounds(65,247,145,40);
		image_scale.setLabelTable(labelTable);
		image_scale.setPaintLabels(true);
		image_scale.setOpaque(false);
		//image_scale.setBackground(new Color(220, 220, 220));
		image_scale.setFocusable(false);
		graphicsPanel.add(image_scale);
		
		image_opa = new JSlider(JSlider.HORIZONTAL,0, 100, 50);
		image_opa.setVisible(false);
		image_opa.setBounds(70,255,135,20);
		graphicsPanel.add(image_opa);
		image_opa.addChangeListener(new ChangeListener() {

			@Override
			public synchronized void stateChanged(ChangeEvent arg0) {
				setImageOpacity(image_opa.getValue());
				System.out.println("Opacity: " + image_opa.getValue());
			}
			
		});
		
		textField = new JTextField();
		textField.setBounds(5,41,135,20);
		graphicsPanel.add(textField);
		
		JButton loadBtn = new JButton("Load");
		loadBtn.setBounds(145, 36, 60, 30);
		loadBtn.addActionListener(uiListener);
		loadBtn.setActionCommand("loadGraphics");
		graphicsPanel.add(loadBtn);
		
		JLabel lblSek_graphics = new JLabel("ms");
		lblSek_graphics.setBounds(173, 195, 22, 16);
		graphicsPanel.add(lblSek_graphics);
		
		JLabel lblms_posx = new JLabel("X");
		lblms_posx.setVisible(false);
		lblms_posx.setBounds(10, 232, 22, 16);
		graphicsPanel.add(lblms_posx);
		
		JLabel lblms_posy = new JLabel("Y");
		lblms_posy.setVisible(false);
		lblms_posy.setBounds(20, 232, 22, 16);
		graphicsPanel.add(lblms_posy);
		
		picBtn = new JButton();
		picBtn.setIcon(new ImageIcon("img/icons/emptygraphic_small.png")); //size 65x65
		picBtn.setBounds(63, 70, 70, 70);
		picBtn.setToolTipText("Move back");
		picBtn.setBorder(BorderFactory.createEmptyBorder(2,0,0,0)); // important
		picBtn.setBorderPainted(false); 
		picBtn.setContentAreaFilled(false); 
		picBtn.setFocusPainted(false); 
		picBtn.setOpaque(false);
		
		picBtn.setActionCommand("pic_preview");
		picBtn.addActionListener(uiListener);
		graphicsPanel.add(picBtn);
		
		model_graphics = new DefaultListModel();
		
		JScrollPane scrollPane_graphic = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane_graphic.setAlignmentY(Component.BOTTOM_ALIGNMENT);
		scrollPane_graphic.setViewportBorder(new LineBorder(Color.LIGHT_GRAY));
		scrollPane_graphic.setBounds(6, 316, 197, 232);;
		scrollPane_graphic.setMaximumSize(new Dimension(190,600));
		scrollPane_graphic.setPreferredSize(new Dimension(190,600));
		graphicsPanel.add(scrollPane_graphic, BorderLayout.CENTER);
		imageList = new JList(model_graphics);
		imageList.setBorder(null);
		ListCellRenderer renderer = new ImageCellRenderer();
	    imageList.setCellRenderer(renderer);
	    addListListener();
		scrollPane_graphic.setViewportView(imageList);
		
		btnGraphicDelete = new JButton("");
		btnGraphicDelete.setBounds(182, 10, 16, 16);
		btnGraphicDelete.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 1));
		btnGraphicDelete.setIcon(new ImageIcon("img/icons/cross.png"));
		btnGraphicDelete.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				uiListener.getImages().deleteImage(imageStore.get(imageList.getSelectedIndex()).getImage());
				updateImageList();
				newGraphic();
			}
			
		});
		graphicsPanel.add(btnGraphicDelete);
		btnGraphicDelete.setVisible(false);
		
		//disable all control items
		setControlsDisabled(false);
	}
	
	

	/*
	 ************************************************************
	 ************************************************************
	 * METHODS USED IN THIS CLASS (ALL)
	 ************************************************************ 
	 ************************************************************
	 */
	/**
	 * Change current tab to @param index.
	 * @param index
	 */
	public void changeTabPane(int index) {
		settingsPane.setSelectedIndex(index);
	}
	
	/**
	 * This method is called at the buildUp 
	 * of the GUI and disables all Panels 
	 * and their items.
	 */
	public void setControlsDisabled(boolean status) {
		Component[] titleCom = this.titlePanel.getComponents();
		for (int i = 0; i < titleCom.length; i++) {
			titleCom[i].setEnabled(status);
		}
		Component[] subtitleCom = this.subtitlePanel.getComponents();
		for (int i = 0; i < subtitleCom.length; i++) {
			subtitleCom[i].setEnabled(status);
		}
		Component[] grahpicsCom = this.graphicsPanel.getComponents();
		for (int i = 0; i < grahpicsCom.length; i++) {
			grahpicsCom[i].setEnabled(status);
		}
	}

	/*
	 ************************************************************
	 ************************************************************
	 * Methods for GraphicsPanel
	 ************************************************************ 
	 ************************************************************
	 */
	
	/**
	 * Instantiate ListSelectionListener for ImageList.
	 */
	ListSelectionListener ll;
	private void addListListener() {
		 ll = new ListSelectionListener() {
			 public void valueChanged(ListSelectionEvent e) { 
				 
				 //sets the currently selected image
				 if(imageList.getSelectedIndex() == 0) {
					   setGraphicPreview(new ImageIcon("img/icons/emptygraphic_small.png")); //set icon for selected image
					   newGraphic();
					   if(textField.getText().equals("")) {
							btnImageEinfgen.setEnabled(false);
					   }
				 }
				 else {
					 System.out.println("List Index: " + imageList.getSelectedIndex());
					 uiListener.getImages().setActiveImageHashMap(imageStore.get(imageList.getSelectedIndex()).getIndex()); 
					 uiListener.getImages().setActiveImageID(imageStore.get(imageList.getSelectedIndex()).getIndex());
					 setGraphicPreview(imageStore.get(imageList.getSelectedIndex()).getIcon()); //set icon for selected image
					 setGraphicText(imageStore.get(imageList.getSelectedIndex()).getImage()); //set values for selected image
				 }
			 }
		}; 
	}
	
	/**
	 * Returns scale of image from JSpinner.
	 * @return
	 */
	public int getImageScale() {
		return this.image_scale.getValue();
	}
	
	/**
	 * Returns duration of image from JTextField.
	 * @return
	 */
	public int getImageDuration() {
		return Integer.parseInt(this.image_duration.getValue().toString());
	}
	
	/**
	 * Returns start of image from JTextfield.
	 * @return
	 */
	public int getImageStart() {
		return Integer.parseInt(this.image_start.getValue().toString());
	}
	
	/**
	 * Returns posx of image from JSpinner.
	 * @return
	 */
	public int getPosX() {
		return Integer.parseInt(this.image_posx.getValue().toString());
	}
	
	/**
	 * Returns posy of image from JSpinner.
	 * @return
	 */
	public int getPosY() {
		return Integer.parseInt(this.image_posy.getValue().toString());
	}
	
	/**
	 * Returns opacity of image.
	 * @unused
	 * @return
	 */
	public int getImageOpacity() {
		return this.imageOpacity;
	}
	
	/**
	 * Sets opacity of image.
	 * @unused
	 * @param imageOpacity
	 */
	public void setImageOpacity(int imageOpacity) {
		this.imageOpacity = imageOpacity;
	}
	
	/**
	 * Used in GraphicsPanel - 
	 * updates the elements shown in JList
	 */
	public void updateImageList() {
		this.imageList.removeListSelectionListener(ll);
		this.imageList.addListSelectionListener(null);

		this.model_graphics = uiListener.getImages().getImageListContent();
		this.imageStore = uiListener.getImages().getImageListStore();
		imageList.setModel(model_graphics);
		this.imageList.addListSelectionListener(ll);
		
		//set preview to standard
		setGraphicPreview(new ImageIcon("img/icons/emptygraphic_small.png")); //set icon for selected image
		newGraphic();
		if(this.textField.getText().equals("")) {
			btnImageEinfgen.setEnabled(false);
		}
	}

	/**
	 * Used in GraphicsPanel
	 * sets PreviewIcon after clicking element in list
	 * @param img
	 */
	public void setGraphicPreview(ImageIcon img){
		img.setImage(img.getImage().getScaledInstance(65, 65, Image.SCALE_FAST));
		this.picBtn.setIcon(img);
	}
	
	/**
	 * Used in Graphics Panel
	 * sets Values after clicking element in list
	 * @param im
	 */
	public void setGraphicText(PlayerImage im) {
		this.image_start.setValue(im.getStart());
		this.image_duration.setValue(im.getDuration());
		this.image_scale.setValue(im.getScale());
		this.image_posx.setValue(im.getPos_x());
		this.image_posy.setValue(im.getPos_y());
		this.textField.setText(im.getPath());
		System.out.println("FilePath: " + im.getPath());
		
		btnImageEinfgen.setText("Update Image");
		btnImageEinfgen.setEnabled(true);
		btnGraphicDelete.setVisible(true);
	}
	
	/**
	 * Sets Path into JTextField.
	 * @param path
	 */
	public void setPathInTextField(String path) {
		this.textField.setText(path);
	}
	
	/**
	 * Returns path set in JTextField.
	 * @param path
	 */
	public String getPathInTextField() {
		return this.textField.getText();
	}
	
	/**
	 * Needed in UIListener::LoadGraphics() to enable Btn
	 * @return
	 */
	public JButton getInsertBtn() {
		return this.btnImageEinfgen;
	}
	
	/**
	 * After deleting element in list set preview to standard (all fields 0).
	 */
	private void newGraphic() {
		this.btnGraphicDelete.setVisible(false);
		this.btnImageEinfgen.setText("Insert Image");
		this.textField.setText("");
		this.image_start.setValue(this.player.getMediaPlayer().getTime());
		this.image_duration.setValue(0);
		this.image_posx.setValue(0);
		this.image_posy.setValue(0);
		this.image_scale.setValue(100);
	}
	
	/*
	 ************************************************************
	 ************************************************************
	 * Methods for SubtitlePanel
	 ************************************************************ 
	 ************************************************************
	 */
	
	/**
	 * Used in SubtitlePanel/GraphicsPanel/TitlePanel.
	 * Video jumps to position "start" when clicking on item in JList.
	 */
	private void jumpToSubtitlePosition() {
		Subtitle selection = this.uiListener.getSubtitles().getSubtitles().get(_currentSub-1);
		float start = selection.getStart();
		float pos = (float)((float)start / (float)this.mainPanel.getMovieDuration());
		
		System.out.println("Set player to position " + pos);
		player.getMediaPlayer().setPosition(pos);
		mainPanel.updatePosition(pos);
		player.getMediaPlayer().setPause(true);
		mainPanel.getControlsPanel().setPaused();
	}
	
	
	/**
	 * Fills the SubtitleList with all subtitles.
	 * @param subs
	 */
	public void fillSubtitleList(ArrayList<Subtitle> subs) {
		this.subtitleList.removeListSelectionListener(this.myListener_subtitle);
		this.model_subtitle.clear();
		this.model_subtitle.add(0, "-- Neuer Untertitel --");
		System.out.println( subs.size());
		for (int i = 0; i < subs.size(); i++) {
			this.model_subtitle.add(i+1, i+" : "+subs.get(i).getValue());
		}
		if(this.myListener_subtitle == null) {
			this.myListener_subtitle = (new ListSelectionListener() {
				 public void valueChanged(ListSelectionEvent e) { 
					 System.out.println(((JList)e.getSource()).getSelectedIndex());
					 editSubtitle(((JList)e.getSource()).getSelectedIndex());
					 _currentSub = ((JList) e.getSource()).getSelectedIndex();
				 }
			});
		}	
		this.subtitleList.addListSelectionListener(this.myListener_subtitle);
	}
	public void emptySubtitleList() {
		if(this.myListener_subtitle != null) this.subtitleList.removeListSelectionListener(this.myListener_subtitle);
		this.model_subtitle.clear();
	}
	
	/**
	 * Edits Subtitles via ActionEvent.
	 * @param e
	 */
	private void editSubtitleAction(ActionEvent e){
		if(this.uiListener.isVideoLoaded) {
			System.out.println(e.getSource().getClass());
			if(this.btn_subtitle_action.getText() == this.text_edit_subtitle){
				this.uiListener.getSubtitles().editSubTitle(this.active_subtitle, 
						(Long.parseLong(String.valueOf(this.subtitle_position.getValue()))), 
						(Long.parseLong(String.valueOf(this.subtitle_position.getValue())))+(Long.parseLong(String.valueOf(this.subtitle_duration.getValue()))), 
						this.subtitle_value.getText());
				this.fillSubtitleList(this.uiListener.getSubtitles().getSubtitles());
			} else if(this.btn_subtitle_action.getText() == this.text_insert_subtitle){
				if(this.uiListener.subtitlesLoaded) {
					this.uiListener.getSubtitles().insertSubTitle(new Subtitle(
						(Long.parseLong(String.valueOf(this.subtitle_position.getValue()))), 
						(Long.parseLong(String.valueOf(this.subtitle_position.getValue())))+(Long.parseLong(String.valueOf(this.subtitle_duration.getValue()))), 
						this.subtitle_value.getText()));
					this.fillSubtitleList(this.uiListener.getSubtitles().getSubtitles());
				} else {
					JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
							"No subtitle-source found.\nYou need to load a Subtitle first.", "Information", JOptionPane.INFORMATION_MESSAGE);
				}
			}
		}
		else {
			JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
					"No video-source found.\nYou need to load a video to add Subtitles.", "Information", JOptionPane.INFORMATION_MESSAGE);
		}	 
	}
	
	
	/**
	 * Edits Subtitles via ID of selected subtitle.
	 * @param id
	 */
	private void editSubtitle(int id) {
		if(id == 0){
			this.newSubtitle();
		} else {
			Subtitle selection = this.uiListener.getSubtitles().getSubtitles().get(id-1);
			this.subtitle_value.setText(selection.getValue());
			this.subtitle_duration.setValue(selection.getEnd()-selection.getStart());
			this.subtitle_position.setValue(selection.getStart());
			this.btn_subtitle_action.setText(this.text_edit_subtitle);
			this.btnSubtitleDelete.setVisible(true);
			
			this.active_subtitle = selection;
		}
	}
	
	/**
	 * Sets standard values for a new subtitle.
	 */
	private void newSubtitle() {
		this.btn_subtitle_action.setText(this.text_insert_subtitle);
		this.subtitle_value.setText("");
		this.subtitle_duration.setValue(0);
		this.subtitle_position.setValue(this.player.getMediaPlayer().getTime());
		this.btnSubtitleDelete.setVisible(false);
	}
	
	
	/*
	 ************************************************************
	 ************************************************************
	 * Methods for TitlePanel
	 ************************************************************ 
	 ************************************************************
	 */
	
	/**
	 * Fills TitleList with all titles.
	 * @param titles
	 */
	public void fillTitleList(ArrayList<Title> titles) {
		this.titleList.removeListSelectionListener(this.myListener_title);
		this.model_title.clear();
		this.model_title.add(0, "-- Neuer Titel --");
		for (int i = 0; i < titles.size(); i++) {
			this.model_title.add(i+1, titles.get(i).getContent());
		}
		if(this.myListener_title == null) {
			System.out.println("listener_init");
			this.myListener_title = (new ListSelectionListener() {
				 public void valueChanged(ListSelectionEvent e) { 
					 System.out.println(((JList)e.getSource()).getSelectedIndex());
					 editTitle(((JList)e.getSource()).getSelectedIndex());
				 }
			});
		}
			
		this.titleList.addListSelectionListener(this.myListener_title);
	}
	
	/**
	 * Edits title information via ActionEvent.
	 * @param e
	 */
	private void editTitleAction(ActionEvent e){
		if(this.uiListener.isVideoLoaded) {
			System.out.println(this.btn_title_action.getText());
			if(this.btn_title_action.getText() == this.text_edit_title){
				this.uiListener.getTitles().editTitle(this.active_title, 
						this.title_content.getText(),//value
						this.title_font.getSelectedIndex(),//font
						this.title_color.getText(),//color
						Integer.valueOf(String.valueOf(this.title_size.getValue())),//size
						(Long.parseLong(String.valueOf(this.title_start.getValue()))),//start	
						(Long.parseLong(String.valueOf(this.title_duration.getValue()))),//duration
						Integer.valueOf(String.valueOf(this.title_x.getValue())), //x
						Integer.valueOf(String.valueOf(this.title_y.getValue()))); //y
				this.fillTitleList(this.uiListener.getTitles().getTitles());
			} else if(this.btn_title_action.getText() == this.text_insert_title){
				System.out.println(this.btn_title_action.getText());
				this.uiListener.getTitles().insertTitle(new Title(
						this.title_content.getText(),//value
						this.title_font.getSelectedIndex(),//font
						this.title_color.getText(),//color
						Integer.valueOf(String.valueOf(this.title_size.getValue())),//size
						(Long.parseLong(String.valueOf(this.title_start.getValue()))),//start	
						(Long.parseLong(String.valueOf(this.title_duration.getValue()))),//duration
						Integer.valueOf(String.valueOf(this.title_x.getValue())), //x
						Integer.valueOf(String.valueOf(this.title_y.getValue()))));
				
				this.fillTitleList(this.uiListener.getTitles().getTitles());
			}
		}
		else {
			JOptionPane.showMessageDialog(this.mainPanel.getMainFrame(),
					"No video-source found.\nYou need to load a video to add Titles.", "Information", JOptionPane.INFORMATION_MESSAGE);
		}	 
	}
	
	/**
	 * Edits title information via ID.
	 * @param id
	 */
	private void editTitle(int id) {
		if(id == 0){
			this.newTitle();
		} else {
			Title selection = this.uiListener.getTitles().getTitles().get(id-1);
			this.title_content.setText(selection.getContent());
			this.title_color.setText(selection.getColor());
			this.title_font.setSelectedIndex(selection.getFont());
			this.title_size.setValue(selection.getSize());
			this.title_x.setValue(selection.getX());
			this.title_y.setValue(selection.getY());
			this.title_start.setValue(selection.getStart());
			this.title_duration.setValue(selection.getDuration());

			this.btn_title_action.setText(this.text_edit_title);
			this.btnTitleDelete.setVisible(true);
			
			this.active_title = selection;
		}
	}
	
	/**
	 * Sets standard values for a new title.
	 */
	private void newTitle() {
		this.btn_title_action.setText(this.text_insert_title);
		this.title_content.setText("");
		this.title_color.setText("#000000");
		this.title_font.setSelectedIndex(0);
		this.title_size.setValue(10);
		this.title_x.setValue(0);
		this.title_y.setValue(0);
		this.title_start.setValue(this.player.getMediaPlayer().getTime());
		this.title_duration.setValue(0);
		this.btnTitleDelete.setVisible(false);
	}
}

