package view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;

import save.SaveVideo;

/**
 * ProgressBar for saving process.
 * Shows the progress of saving and converting the video.
 * @author Johann Houszka 0625523
 *
 */
public class StatusDialog extends JDialog implements ActionListener{

	/**
	 * Default Serial ID
	 */
	private static final long serialVersionUID = 1L;
	
	private long length;
	private PlayerMainPanel mainPanel;
	JTextArea msgLabel;
    JProgressBar progressBar;
    JPanel panel, subPanel;
    JButton btnok, btncancel;
    int MAXIMUM;
    private SaveVideo sv;
	    
    /**
     * Constructor for this class.
     * @param mainPanel
     * @param length
     * @param sv
     */
    public StatusDialog(PlayerMainPanel mainPanel, long length, SaveVideo sv) {
    	
    	this.length = length;
        this.mainPanel = mainPanel;	
    	this.sv = sv;
    	
        MAXIMUM = (int) length;
    	init(this.length);
    	
    }
    
	public StatusDialog(PlayerMainPanel mainPanel, long length) {
	    	
	    	this.length = length;
	        this.mainPanel = mainPanel;	
	    	
	        MAXIMUM = (int) length;
	    	initOpen(this.length);
	    	
	}
    
    /**
     * Updates info on progressbar.
     * @param message
     * @param value
     */
    public void updateInfo(String message, long value){
    	int v = (int)value;
    	progressBar.setValue(v);
    	msgLabel.setText(message);
    	
    	if(v == MAXIMUM) {
    		btnok.setEnabled(true);
    	}
    }
    
    /**
     * Updates info on progressbar.
     * For indeterminate view.
     * @param message
     * @param value
     */
    public void updateInfo(String message){
    	msgLabel.setText(message);
    }
 
    /**
     * Builds the GUI for saving 
     * @param length
     */
    public void init(long length) {
        progressBar = new JProgressBar(0, MAXIMUM);
        progressBar.setIndeterminate(false); //type of progressbar
        msgLabel = new JTextArea("Work Progress");
        msgLabel.setEditable(false);
        btnok = new JButton("OK");
        btnok.setSize(new Dimension(60,30));
        btnok.addActionListener(this);
        btnok.setActionCommand("btnok");
        btnok.setEnabled(false);
        btncancel = new JButton("Cancel");
        btncancel.setSize(new Dimension(60,30));
        btncancel.addActionListener(this);
        btncancel.setActionCommand("btncancel");
        
        subPanel = new JPanel(new BorderLayout());
        subPanel.add(btnok, BorderLayout.EAST);
        subPanel.add(btncancel, BorderLayout.WEST);
        
        panel = new JPanel(new BorderLayout(5, 5));
        panel.add(msgLabel, BorderLayout.PAGE_START);
        panel.add(progressBar, BorderLayout.CENTER);
        panel.add(subPanel, BorderLayout.PAGE_END);
        panel.setBorder(BorderFactory.createEmptyBorder(11, 11, 11, 11));

        getContentPane().add(panel);
        setResizable(false);
        pack();
        setSize(300,getHeight());
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        setAlwaysOnTop(false);
        setVisible(true);
        msgLabel.setBackground(panel.getBackground());
    }
    
    /**
     * Builds the GUI for opening 
     * @param length
     */
    public void initOpen(long length) {
        progressBar = new JProgressBar(0, MAXIMUM);
        progressBar.setIndeterminate(true); //type of progressbar
        msgLabel = new JTextArea("Work Progress");
        msgLabel.setEditable(false);
        
        panel = new JPanel(new BorderLayout(5, 5));
        panel.add(msgLabel, BorderLayout.PAGE_START);
        panel.add(progressBar, BorderLayout.CENTER);
        
        panel.setBorder(BorderFactory.createEmptyBorder(11, 11, 11, 11));

        getContentPane().add(panel);
        setResizable(false);
        pack();
        setSize(300,getHeight());
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        setAlwaysOnTop(false);
        setVisible(true);
        msgLabel.setBackground(panel.getBackground());
    }

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("btnok")) {
			dispose();
		}
		if(e.getActionCommand().equals("btncancel")) {
			this.sv.stopSavingProcess();
			try {
				Thread.sleep(500);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			dispose();
		}
		
	}
    
}
